def abbr(self, abbr="", string="", **kwargs):
    """Defines an abbreviation.

    APDL Command: *ABBR

    Parameters
    ----------
    abbr
        The abbreviation (up to 8 alphanumeric characters) used to
        represent the string String.  If Abbr is the same as an existing
        ANSYS command, the abbreviation overrides.  Avoid using an Abbr
        which is the same as an ANSYS command.

    string
        String of characters (60 maximum) represented by Abbr.  Cannot
        include a $ or any of the commands C***,  /COM,  /GOPR,  /NOPR,
        /QUIT,  /UI, or  *END.  Parameter names and commands of the *DO and
        Use the *IF groups may not be abbreviated. If String is blank, the
        abbreviation is deleted. To abbreviate multiple commands, create an
        "unknown command" macro or define String to execute a macro file
        [*USE] containing the desired commands.

    Notes
    -----
    Once the abbreviation Abbr is defined, you can issue it at the
    beginning of a command line and follow it with a blank (or with a comma
    and appended data), and the program will substitute the string  String
    for Abbr as the line is executed. Up to 100 abbreviations may exist at
    any time and are available throughout the program. Abbreviations may be
    redefined or deleted at any time.

    Use *STATUS to display the current list of abbreviations. For
    abbreviations repeated with *REPEAT, substitution occurs before the
    repeat increments are applied. There are a number of abbreviations that
    are predefined by the program (these can be deleted by using the blank
    String option described above). Note that String will be written to the
    File.LOG.

    This command is valid in any processor.
    """
    command = f"*ABBR,{abbr},{string}"
    return self.run(command, **kwargs)


def abbres(self, lab="", fname="", ext="", **kwargs):
    """Reads abbreviations from a coded file.

    APDL Command: ABBRES

    Parameters
    ----------
    lab
        Label that specifies the read operation:

        Replace current abbreviation set with these abbreviations (default). - Extend current abbreviation set with these abbreviations, replacing any of the
                          same name that already exist.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    The abbreviation file may have been written with the ABBSAV command. Do
    not issue ABBRES,NEW while inside an executing abbreviation. Doing so
    will cause all data for the executing abbreviation to be deleted.

    This command is valid in any processor.
    """
    command = f"ABBRES,{lab},{fname},{ext}"
    return self.run(command, **kwargs)


def abbsav(self, lab="", fname="", ext="", **kwargs):
    """Writes the current abbreviation set to a coded file.

    APDL Command: ABBSAV

    Parameters
    ----------
    lab
        Label that specifies the write operation.

        ALL - Write all abbreviations (default).

    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

        The file name defaults to Jobname.

    ext
        Filename extension (eight-character maximum).  The
        extension defaults to ABBR if Fname is blank.

    Notes
    -----
    Existing abbreviations on this file, if any, will be overwritten.  The
    abbreviation file may be read with the ABBRES command.

    This command is valid in any processor.
    """
    command = f"ABBSAV,{lab},{fname},{ext}"
    return self.run(command, **kwargs)


def ucmd(self, cmd="", srnum="", **kwargs):
    """Assigns a user-defined command name.

    APDL Command: /UCMD

    Parameters
    ----------
    cmd
        User-defined command name.  Only the first four characters are
        significant.  Must not conflict with any ANSYS command name or any
        user "unknown command" macro name.

    srnum
        User subroutine number (1 to 10) programmed for this command.  For
        example, the command /UCMD,MYCMD,3 will execute subroutine USER03
        whenever the command MYCMD is entered.  Use a blank command name to
        disassociate SRNUM from its command.  For example, /UCMD,,3 removes
        MYCMD as a command.

    Notes
    -----
    Assigns a user-defined command name to a user-programmable (system-
    dependent) subroutine.  This feature allows user-defined commands to be
    programmed into the ANSYS program.  Once programmed, this command can
    be input to the program like other commands, and can also be included
    in the ANSYS start-up file.  See *ULIB for another way of defining user
    commands.

    Up to 10 subroutines are available for user-defined commands (USER01 to
    USER10).  Users must have system permission, system access, and
    knowledge to write, compile, and link the appropriate subprocessors
    into the ANSYS program at the site where it is to be run.  All routines
    should be written in FORTRAN. For more information on FORTRAN compilers
    please refer to either the ANSYS, Inc. Windows Installation Guide or
    the ANSYS, Inc. Linux Installation Guide for details specific to your
    platform or operating system. The USER01 routine is commented and
    should be listed from the distribution media (system dependent) for
    more details.  Issue /UCMD,STAT to list all user-defined command names.
    Since a user-programmed command is a nonstandard use of the program,
    the verification of any ANSYS run incorporating these commands is
    entirely up to the user.  In any contact with ANSYS customer support
    regarding the performance of a custom version of the ANSYS program, you
    should explicitly state that a user programmable feature has been used.
    See the Advanced Analysis Guide for a general description of user-
    programmable features and Guide to User-Programmable Features for a
    detailed description of these features.

    This command is valid only at the Begin Level.
    """
    command = f"/UCMD,{cmd},{srnum}"
    return self.run(command, **kwargs)



def bf(self, node="", lab="", val1="", val2="", val3="", val4="", val5="",
       val6="", **kwargs):
    """Defines a nodal body force load.

    APDL Command: BF

    Parameters
    ----------
    node
        Node to which body load applies.  If Node = ALL, apply to all
        selected nodes [NSEL].  A component name may also be substituted
        for Node.

    lab
        Valid body load label. Load labels are listed under "Body Loads" in
        the input table for each element type in the Element Reference.

    val1, val2, val3, val4, val5, val6
        Value associated with the Lab item or table name reference for
        tabular boundary conditions. To specify a table, enclose the table
        name in percent signs (%) (e.g., BF,Node,TEMP,%tabname%). Use the
        *DIM command to define a table. Use only VAL1 for TEMP, FLUE, HGEN,
        DGEN, MVDI, CHRGD. If Lab = CHRGD for acoustics, VAL1 is the static
        pressure for a non-uniform acoustic medium calculation.

        VAL1 - Mass source with units kg/(m3s) in a harmonic analysis, or mass source rate
               with units kg/( m3s2) in a transient analysis

        VAL2 - Phase angle in degrees

        VAL3 - Not used

        VAL4 - Not used

        VAL5 - Not used

        VAL6 - Not used

    Notes
    -----
    Defines a nodal body force load (such as temperature in a structural
    analysis, heat generation rate in a thermal analysis, etc.). Nodal body
    loads default to the BFUNIF values, if they were previously specified.

    Table names are valid for Lab value (VALn) inputs in these cases only:

    VAL = %tabname% for temperatures (TEMP), diffusing substance generation
    rates (DGEN), heat generation rates (HGEN), and nodal body force
    densities (FORC).

    VAL1 = %tabname1%, VAL2 = %tabname2%, VAL3 = %tabname3%, VAL4 =
    %tabname4%, VAL5 = %tabname5%, and VAL6 = %tabname6% for velocities or
    accelerations (VELO).

    VAL1 = %tabname1% and VAL2 = %tabname2% for mass sources or mass source
    rates (MASS).

    The heat generation rate loads specified with the BF command are
    multiplied by the weighted nodal volume of each element adjacent to
    that node. This yields the total heat generation at that node.

    Graphical picking is available only via the listed menu paths.

    Body load labels VELO and MASS cannot be accessed from a menu.

    This command is also valid in PREP7.
    """
    command = f"BF,{node},{lab},{val1},{val2},{val3},{val4},{val5},{val6}"
    return self.run(command, **kwargs)


def bfcum(self, lab="", oper="", fact="", tbase="", **kwargs):
    """Specifies that nodal body force loads are to be accumulated.

    APDL Command: BFCUM

    Parameters
    ----------
    lab
        Valid body load label.  If ALL, use all appropriate labels.

    oper
        Accumulation key:

        REPL - Subsequent values replace the previous values (default).

        ADD - Subsequent values are added to the previous values.

        IGNO - Subsequent values are ignored.

    fact
        Scale factor for the nodal body load values.  Zero (or blank)
        defaults to 1.0.  Use a small number for a zero scale factor.  The
        scale factor is not applied to body load phase angles.

    tbase
        Used (only with Lab = TEMP) to calculate the temperature used in
        the add or replace operation (see Oper) as:

    Notes
    -----
    Allows repeated nodal body force loads to be replaced, added, or
    ignored.  Nodal body loads are applied with the BF command.  Issue the
    BFLIST command to list the nodal body loads.  The operations occur when
    the next body loads are defined.  For example, issuing the BF command
    with a temperature of 250 after a previous BF command with a
    temperature of 200 causes the new value of the temperature to be 450
    with the add operation, 250 with the replace operation, or 200 with the
    ignore operation.  A scale factor is also available to multiply the
    next value before the add or replace operation.  A scale factor of 2.0
    with the previous "add" example results in a temperature of 700.  The
    scale factor is applied even if no previous values exist.  Issue
    BFCUM,STAT to show the current label, operation, and scale factors.
    Solid model boundary conditions are not affected by this command, but
    boundary conditions on the FE model are affected.

    Note:: : FE boundary conditions may still be overwritten by existing
    solid model boundary conditions if a subsequent boundary condition
    transfer occurs.

    BFCUM does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"BFCUM,{lab},{oper},{fact},{tbase}"
    return self.run(command, **kwargs)


def bfdele(self, node="", lab="", **kwargs):
    """Deletes nodal body force loads.

    APDL Command: BFDELE

    Parameters
    ----------
    node
        Node at which body load is to be deleted.  If ALL, delete for all
        selected nodes [NSEL].  If NODE = P, graphical picking is enabled
        and all remaining command fields are ignored (valid only in the
        GUI).  A component name may also be substituted for NODE.

    lab
        Valid body load label. If ALL, use all appropriate labels. See the
        BF command for labels. In an explicit dynamic analysis, the only
        valid body load label is TEMP.

    Notes
    -----
    Deletes nodal body force loads for a specified node and label.  Nodal
    body loads may be defined with the BF command (except in an explicit
    dynamic analysis).

    The command BFDELE,TEMP can be used in an explicit dynamic analysis to
    delete temperature loads that are read in by the LDREAD command. BFDELE
    cannot be used to delete temperature loads defined by the EDLOAD
    command (use EDLOAD,DELE to delete this type of load).

    This command is also valid in PREP7.
    """
    command = f"BFDELE,{node},{lab}"
    return self.run(command, **kwargs)


def bfe(self, elem="", lab="", stloc="", val1="", val2="", val3="",
        val4="", **kwargs):
    """Defines an element body force load.

    APDL Command: BFE

    Parameters
    ----------
    elem
        The element to which body load applies.  If ALL, apply to all
        selected elements (ESEL). A component name may also be substituted
        for Elem.

    lab
        Valid body load label. Valid labels are also listed for each
        element type in the Element Reference under "Body Loads" in the
        input table.

    stloc
        Starting location for entering VAL data, below.  For example, if
        STLOC = 1, data input in the VAL1 field applies to the first
        element body load item available for the element type, VAL2 applies
        to the second element item, etc. If STLOC = 5, data input in the
        VAL1 field applies to the fifth element item, etc.  Defaults to 1.

    val1, val2, val3, val4
        For Lab = TEMP, FLUE, DGEN, HGEN, and CHRGD, VAL1--VAL4 represent
        body load values at the starting location and subsequent locations
        (usually nodes) in the element.  VAL1 can also represent a table
        name for use with tabular boundary conditions. Enter only VAL1 for
        a uniform body load across the element.  For nonuniform loads, the
        values must be input in the same order as shown in the input table
        for the element type.  Values initially default to the BFUNIF value
        (except for CHRGD which defaults to zero).  For subsequent
        specifications, a blank leaves a previously specified value
        unchanged; if the value was not previously specified, the default
        value as described in the Element Reference is used.

    Notes
    -----
    Defines an element body force load (such as temperature in a structural
    analysis, heat generation rate in a thermal analysis, etc.). Body loads
    and element specific defaults are described for each element type in
    the Element Reference. If both the BF and BFE commands are used to
    apply a body load to an element, the BFE command takes precedence.

    For heat-generation (HGEN) loading on layered thermal solid elements
    SOLID278 / SOLID279 (KEYOPT(3) = 1 or 2), or layered thermal shell
    elements SHELL131 / SHELL132 (KEYOPT(3) = 1), STLOC refers to the layer
    number (not the node). In such cases, use VAL1 through VAL4 to specify
    the heat-generation values for the appropriate layers. Heat generation
    is constant over the layer.

    Specifying a Table

    You can specify a table name (VAL1) when using temperature (TEMP),
    diffusing substance generation rate (DGEN), heat generation rate
    (HGEN), and current density (JS) body load labels.

    Enclose the table name (tabname) in percent signs (%), as shown:

    BFE,Elem, Lab,STLOC,%tabname%

    Use the *DIM command to define a table.

    For Lab = TEMP, each table defines NTEMP temperatures, as follows:

    For layered elements, NTEMP is the number of layer interface corners
    that allow temperature input.

    For non-layered elements, NTEMP is the number of corner nodes.

    The temperatures apply to element items with a starting location of
    STLOC + n, where n is the value field location (VALn) of the table name
    input.

    For layered elements, a single BFE command returns temperatures for one
    layer interface. Multiple BFE commands are necessary for defining all
    layered temperatures.

    For beam, pipe and elbow elements that allow multiple temperature
    inputs per node, define the tabular load for the first node only (Node
    I), as loads on the remaining nodes are applied automatically. For
    example, to specify a tabular temperature load on a PIPE288 element
    with the through-wall-gradient option (KEYOPT(1) = 0), the BFE command
    looks like this:

    BFE,Elem,TEMP,1,%tabOut%, %tabIn%

    When a tabular function load is applied to an element, the load does
    not vary according to the positioning of the element in space.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFE,{elem},{lab},{stloc},{val1},{val2},{val3},{val4}"
    return self.run(command, **kwargs)


def bfecum(self, lab="", oper="", fact="", tbase="", **kwargs):
    """Specifies whether to ignore subsequent element body force loads.

    APDL Command: BFECUM

    Parameters
    ----------
    lab
        Valid body load label.  If ALL, use all appropriate labels.

    oper
        Replace or ignore key:

        REPL - Subsequent values replace the previous values (default).

        IGNO - Subsequent values are ignored.

    fact
        Scale factor for the element body load values.  Zero (or blank)
        defaults to 1.0.  Use a small number for a zero scale factor.  The
        scale factor is not applied to body load phase angles.

    tbase
        Used (only with  Lab = TEMP) to calculate the temperature used in
        the add or replace operation (see Oper) as:

    Notes
    -----
    Allows repeated element body force loads to be replaced or ignored.
    Element body loads are applied with the BFE command.  Issue the BFELIST
    command to list the element body loads.  The operations occur when the
    next body loads are defined.  For example, issuing the BFE command with
    a temperature value of 25 after a previous BFE command with a
    temperature value of 20 causes the new value of that temperature to be
    25 with the replace operation, or 20 with the ignore operation.  A
    scale factor is also available to multiply the next value before the
    replace operation.  A scale factor of 2.0 with the previous "replace"
    example results in a temperature of 50.  The scale factor is applied
    even if no previous values exist.  Issue BFECUM,STAT to show the
    current label, operation, and scale factors.

    BFECUM does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"BFECUM,{lab},{oper},{fact},{tbase}"
    return self.run(command, **kwargs)


def bfedele(self, elem="", lab="", **kwargs):
    """Deletes element body force loads.

    APDL Command: BFEDELE

    Parameters
    ----------
    elem
        Element at which body load is to be deleted.  If ALL, delete for
        all selected elements [  A component name may also be substituted
        for ELEM.

    lab
        Valid body load label. If ALL, use all appropriate labels. See BFE
        command for labels.

    Notes
    -----
    Deletes element body force loads for a specified element and label.
    Element body loads may be defined with the BFE commands.

    Graphical picking is available only via the listed menu paths.

    This command is also valid in PREP7.
    """
    command = f"BFEDELE,{elem},{lab}"
    return self.run(command, **kwargs)


def bfelist(self, elem="", lab="", **kwargs):
    """Lists the element body force loads.

    APDL Command: BFELIST

    Parameters
    ----------
    elem
        Element at which body load is to be listed.  If ALL (or blank),
        list for all selected elements [ESEL].  If ELEM = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may also be substituted
        for ELEM.

    lab
        Valid body load label. If ALL (or blank), use all appropriate
        labels. See BFE command for labels.

    Notes
    -----
    Lists the element body force loads for the specified element and label.
    Element body loads may be defined with the BFE command.

    This command is valid in any processor.
    """
    command = f"BFELIST,{elem},{lab}"
    return self.run(command, **kwargs)


def bfescal(self, lab="", fact="", tbase="", **kwargs):
    """Scales element body force loads.

    APDL Command: BFESCAL

    Parameters
    ----------
    lab
        Valid body load label.  If ALL, use all appropriate labels.

    fact
        Scale factor for the element body load values.  Zero (or blank)
        defaults  to 1.0.  Use a small number for a "zero" scale factor.
        The scale factor is not applied to body load phase angles.

    tbase
        Base temperature for temperature difference.  Used only with Lab =
        TEMP.  Scale factor is applied to the temperature difference (T -
        TBASE)  and then added to TBASE.  T is the current temperature.

    Notes
    -----
    Scales element body force loads on the selected elements in the
    database.  Issue the BFELIST command to list the element body loads.
    Solid model boundary conditions are not scaled by this command, but
    boundary conditions on the FE model are scaled.  (Note that such scaled
    FE boundary conditions may still be overwritten by unscaled solid model
    boundary conditions if a subsequent boundary condition transfer
    occurs.)

    BFESCAL does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"BFESCAL,{lab},{fact},{tbase}"
    return self.run(command, **kwargs)


def bflist(self, node="", lab="", **kwargs):
    """Lists the body force loads on nodes.

    APDL Command: BFLIST

    Parameters
    ----------
    node
        Node at which body load is to be listed.  If ALL (or blank), list
        for all selected nodes [NSEL].  If NODE = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for NODE.

    lab
        Valid body load label. If ALL (or blank), use all appropriate
        labels. See the BF command for labels. In an explicit dynamic
        analysis, the only valid body load label is TEMP.

    Notes
    -----
    Lists the body force loads for the specified node and label.  Nodal
    body loads may be defined with the BF command (except in an explicit
    dynamic analysis).

    The command BFLIST,TEMP can be used in an explicit dynamic analysis to
    list temperature loads that are read in by the LDREAD command. BFLIST
    cannot be used to list temperature loads defined by the EDLOAD command
    (use EDLOAD,LIST to list this type of load).

    This command is valid in any processor.
    """
    command = f"BFLIST,{node},{lab}"
    return self.run(command, **kwargs)


def bfscale(self, lab="", fact="", tbase="", **kwargs):
    """Scales body force loads at nodes.

    APDL Command: BFSCALE

    Parameters
    ----------
    lab
        Valid body load label.  If ALL, use all appropriate labels.

    fact
        Scale factor for the nodal body load values.  Zero (or blank)
        defaults  to 1.0.  Use a small number for a zero scale factor.  The
        scale factor is not applied to body load phase angles.

    tbase
        Base temperature for temperature difference.  Used only with Lab =
        TEMP.  Scale factor is applied to the temperature difference (T -
        TBASE)  and then added to TBASE.  T is the current temperature.

    Notes
    -----
    Scales body force loads in the database on the selected nodes.  Issue
    the BFLIST command to list the nodal body loads.  Solid model boundary
    conditions are not scaled by this command, but boundary conditions on
    the FE model are scaled.

    Note:: : Such scaled FE boundary conditions may still be overwritten by
    unscaled solid model boundary conditions if a subsequent boundary
    condition transfer occurs.

    BFSCALE does not work for tabular boundary conditions.

    This command is also valid in PREP7.
    """
    command = f"BFSCALE,{lab},{fact},{tbase}"
    return self.run(command, **kwargs)


def bfunif(self, lab="", value="", **kwargs):
    """Assigns a uniform body force load to all nodes.

    APDL Command: BFUNIF

    Parameters
    ----------
    lab
        Valid body load label.  If ALL, use all appropriate labels.

    value
        Uniform value associated with Lab item, or table name when
        specifying tabular boundary conditions.  To specify a table,
        enclose the table name in percent signs (%), e.g.,
        BFUNIF,Lab,%tabname%.

    Notes
    -----
    In a transient or nonlinear thermal analysis, the uniform temperature
    is used during the first iteration of a solution as follows:  (a) as
    the starting nodal temperature (except where temperatures are
    explicitly specified [D, DK]), and (b) to evaluate temperature-
    dependent material properties.  In a structural analysis or explicit
    dynamic analysis, the uniform temperature is used as the default
    temperature for thermal strain calculations and material property
    evaluation (except where body load temperatures are specified [BF, BFE,
    BFK, LDREAD]).  In other scalar field analyses, the uniform temperature
    is used for material property evaluation.

    When the command BFUNIF,TEMP is used in an explicit dynamic analysis,
    you cannot use the EDLOAD,TEMP command to apply temperature loading.
    Furthermore, any temperature loading defined by BFUNIF cannot be listed
    or deleted by the EDLOAD command.

    An alternate command, TUNIF, may be used to set the uniform temperature
    instead of BFUNIF,TEMP. Since TUNIF (or BFUNIF,TEMP) is step-applied in
    the first iteration, you should use BF, ALL, TEMP, Value to ramp on a
    uniform temperature load.

    You can specify a table name only when using temperature (TEMP), heat
    generation rate (HGEN), and diffusing substance generation rate (DGEN)
    body load labels. When using TEMP, you can define a one-dimensional
    table that varies with respect to time (TIME) only. When defining this
    table, enter TIME as the primary variable. No other primary variables
    are valid. Tabular boundary conditions cannot be used in an explicit
    dynamic analysis.

    This command is also valid in PREP7.
    """
    command = f"BFUNIF,{lab},{value}"
    return self.run(command, **kwargs)


def ldread(self, lab="", lstep="", sbstep="", time="", kimg="", fname="",
           ext="", **kwargs):
    """Reads results from the results file and applies them as loads.

    APDL Command: LDREAD

    Parameters
    ----------
    lab
        Valid load label:

        TEMP - Temperatures from a thermal analysis are applied as body force nodal loads (BF)
               in a structural analysis, an explicit dynamic analysis, or
               other type of analysis.

        If the thermal analysis uses SHELL131 or SHELL132, temperatures are applied as body force element loads (BFE). In most cases, only the top and bottom temperatures from SHELL131 and SHELL132 are used by the structural shell elements; any interior temperatures are ignored.  - All temperatures are used for SHELL181 using section input, and SHELL281 using
                          section input; for these elements, therefore, the
                          number of temperature points at a node generated
                          in the thermal model must match the number of
                          temperature points at a node needed by the
                          structural model.

        When using SHELL131 or SHELL132 information for the LDREAD operation, all element types should specify the same set of thermal degrees of freedom. - When used in conjunction with KIMG=1 and KIMG=2, temperatures can be applied to
                          a subsequent thermal analysis as nodal loads (D)
                          or initial conditions (IC), respectively.

        FORC - Forces from an electromagnetic analysis are applied as force loads (F) in a
               structural analysis. LDREAD,FORC reads coupling forces. See
               the discussion on force computation in the Low-Frequency
               Electromagnetic Analysis Guide.

          For a full harmonic magnetic analysis, FORC represents the time-averaged force (use in conjunction with KIMG = 2).  Values are in the nodal coordinate system for the force loads (F). - HGEN

        Heat generations from an electromagnetic analysis are applied as body force loads (BFE) in a thermal analysis.  For a full harmonic analysis, HGEN represents the time-averaged heat generation load (use in conjunction with KIMG = 2). - JS

        Source current density from a current-conduction analysis are applied as body force loads (BFE).  Values are in the global Cartesian coordinate system. - EF

        Electric field element centroid values from an electrostatic analysis are applied as body force loads (BFE) in a magnetic analysis. Values are in the global Cartesian coordinate system. - REAC

        Reaction loads from any analysis are applied as force loads (F) in any analysis.  Values are in the nodal coordinate system. - CONC

    lstep
        Load step number of the data set to be read.  Defaults to 1.  If
        LAST, ignore SBSTEP and TIME and read the last data set.

    sbstep
        Substep number (within LSTEP).  If zero (or blank), LSTEP
        represents the last substep of the load step.

    time
        Time-point identifying the data set to be read.  Used only if both
        LSTEP and SBSTEP are zero (or blank).  If TIME is between two
        solution time points on the results file, a linear interpolation is
        done between the two data sets.  If TIME is beyond the last time
        point on the file, use the last time point.

    kimg
        When used with results from harmonic analyses (ANTYPE,HARMIC) KIMG
        establishes which set of data to read:

        0 - Read the real part of the solution.  Valid also for Lab = EHFLU to read in
            time-average heat flux.

        1 - Read the imaginary part of the solution.

        2 - Calculate and read the time-average part. Meaningful for Lab = HGEN or FORC.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    The LDREAD command reads results data from the results file and applies
    them as loads.

    The command can also apply results from an analysis defined with one
    physics environment as loads on a second analysis using a different
    physics environment. Results values are applied as loads for field-
    coupling effects (for example, output temperatures from a thermal
    analysis as input to a structural analysis).

    The command works based on the assumption that the meshes have not
    changed.

    Nodal loads are applied only to selected nodes. Element loads are
    applied only to selected elements. Element surface loads are applied
    only to selected elements where all face nodes for that surface are
    selected.

    To assure proper distribution of the surface loads, select only the
    nodes on the element face where the surface load is to be applied.

    Scaling and accumulation specifications are applied as the loads are
    read via the following commands:

    BFCUM for body force loads. (Heat-generation loads are not
    accumulated.)
    """
    command = f"LDREAD,{lab},{lstep},{sbstep},{time},{kimg},{fname},{ext}"
    return self.run(command, **kwargs)


def rimport(self, source="", type_="", loc="", lstep="", sbstep="",
            fname="", ext="", spscale="", mscale="", **kwargs):
    """Imports initial stresses from an explicit dynamics run into ANSYS.

    APDL Command: RIMPORT

    Parameters
    ----------
    source
        The type of analysis run from which stresses are imported.

        OFF - Ignore initial stresses.

        DYNA - Get initial stresses from an earlier explicit
        (ANSYS LS-DYNA) run (default).

    type\_
        Type of data imported.  Note that this is an ANSYS-defined
        field; the only valid value is STRESS.

    loc
        Location where the data is imported.  Note that this is an
        ANSYS- defined field; the only valid value is ELEM (data
        imported at the element integration points).

    lstep
        Load step number of data to be imported.  Defaults to the
        last load step.

    sbstep
        Substep number of data to be imported.  Defaults to the
        last substep.

    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    spscale
        Stabilization factor. This factor is used in a springback
        analysis to scale (up or down) the initial stiffness of
        the applied spring.  No default; input a value only if you
        want to activate stabilization. If SPSCALE is blank,
        stabilization is not activated.

    mscale
        Acceptable stabilization stiffness (defaults to 1.0 X
        10--4). In a springback analysis, iterations will stop
        when the applied spring stiffness comes down to this
        value. MSCALE is not used if SPSCALE is blank.

    Notes
    -----
    This command imports initial stress information into ANSYS
    from an earlier explicit (ANSYS LS-DYNA) run.  The stress
    state from SHELL163 and SOLID164 elements in the explicit
    analysis is imported to the corresponding SHELL181 and
    SOLID185 implicit elements. For the shell elements, the
    current shell element thickness is also imported. This command
    is valid only before the first SOLVE command of the implicit
    analysis (which comes after the explicit analysis) and is
    ignored if issued after subsequent SOLVE commands (that is,
    stresses will not be re-imported).

    RIMPORT is typically used to perform springback analysis of
    sheet metal forming. We recommend that you use SHELL163
    elements in the explicit analysis with 3 to 5 integration
    points through the thickness. This ensures that the
    through-thickness stress distribution is transferred
    accurately to the SHELL181 elements. If more than 5
    integration points are used, ANSYS imports resultants (forces
    and moments) to the SHELL181 elements. This implies that
    linearization of the through-thickness stress distribution is
    assumed in SHELL181 elements. If SHELL163 uses full
    integration in the shell plane, stress and thickness data are
    averaged and then transferred. For the solid elements, the
    stress at the SOLID164 element centroid is transferred to the
    SOLID185 element centroid. If SOLID164 has full integration,
    the stress is averaged and then transferred.

    When the SPSCALE argument is specified, artificial springs
    with exponentially decaying stiffness (as a function of
    iterations) are applied. This technique is recommended only
    for those cases in which there are severe convergence
    difficulties. In general, you should first attempt a
    springback analysis without using the stabilization factors
    SPSCALE and MSCALE. (For more information on springback
    stabilization, see the ANSYS LS-DYNA User's Guide.)

    This command is not written to the Jobname.CDB file when the
    CDWRITE command is issued. Further, the RIMPORT information is
    not saved to the database; therefore, the RIMPORT command must
    be reissued if the database is resumed.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"RIMPORT,{source},{type_},{loc},{lstep},{sbstep},{fname},{ext},,{spscale},{mscale}"
    return self.run(command, **kwargs)


def tunif(self, temp="", **kwargs):
    """Assigns a uniform temperature to all nodes.

    APDL Command: TUNIF

    Parameters
    ----------
    temp
        Uniform temperature assigned to the nodes. If a TEMP value is not
        specified, the uniform temperature is set to zero.

    Notes
    -----
    TUNIF is a convenient form of the more general BFUNIF command.

    In a transient or nonlinear thermal analysis, the uniform temperature
    is used during the first iteration of a solution as follows:

    as the starting nodal temperature (except where temperatures are
    explicitly specified [D, DK]),

    to evaluate temperature-dependent material properties.

    In a structural analysis or an explicit dynamic analysis, the uniform
    temperature is used as the default temperature for thermal strain
    calculations and material property evaluation (except where body load
    temperatures are specified (BF, BFE, BFK, LDREAD). In other scalar
    field analyses, the uniform temperature is used for material property
    evaluation.

    Because TUNIF (or BFUNIF,TEMP) is step-applied in the first iteration,
    issue a BF,ALL,TEMP,Value command to ramp on a uniform temperature
    load.

    When the TUNIF command is used in an explicit dynamic analysis, you
    cannot apply temperature loading via the EDLOAD,,TEMP command.
    Furthermore, temperature loading defined by TUNIF cannot be listed or
    deleted by the EDLOAD command.

    The command default sets the uniform temperature to the reference
    temperature defined via the TREF command only (and not the MP,REFT
    command).

    If using the command default to set the uniform temperature (to the
    reference temperature set via TREF), you can convert temperature-
    dependent secant coefficients of thermal expansion (SCTEs) from the
    definition temperature to the uniform temperature. To do so, issue the
    MPAMOD command.

    This command is also valid in PREP7.
    """
    command = f"TUNIF,{temp}"
    return self.run(command, **kwargs)



def rmresume(self, fname="", ext="", **kwargs):
    """Resumes ROM data from a file.

    APDL Command: RMRESUME

    Parameters
    ----------
    fname
        Name and directory path of the ROM database file (248 character
        maximum). Default to Jobname.

    ext
        Extension of the ROM database file. Default to .rom.

    Notes
    -----
    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"RMRESUME,{fname},{ext}"
    return self.run(command, **kwargs)


def rmsave(self, fname="", ext="", **kwargs):
    """Saves ROM data to file.

    APDL Command: RMSAVE

    Parameters
    ----------
    fname
        Name and directory path of the ROM database file. Default to
        Jobname.

    ext
        Extension of the ROM database file. Default to .rom.

    Notes
    -----
    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    return self.run(f"RMSAVE,{fname},{ext}", **kwargs)



"""These DATABASE commands define and manipulate coordinate systems."""


def clocal(self, kcn="", kcs="", xl="", yl="", zl="", thxy="", thyz="",
           thzx="", par1="", par2="", **kwargs):
    """Defines a local coordinate system relative to the active coordinate

    APDL Command: CLOCAL
    system.

    Parameters
    ----------
    kcn
        Arbitrary reference number assigned to this coordinate system.
        Must be greater than 10.  A coordinate system previously defined
        with this number will be redefined.

    kcs
        Coordinate system type:

        0 or CART - Cartesian

        1 or CYLIN - Cylindrical (circular or elliptical)

        2 or SPHE - Spherical (or spheroidal)

        3 or TORO - Toroidal

    xl, yl, zl
        Location (in the active coordinate system) of the origin of the new
        coordinate system (R, θ, Z for cylindrical, R, θ,Φ for spherical or
        toroidal).

    thxy
        First rotation about local Z (positive X toward Y).

    thyz
        Second rotation about local X (positive Y toward Z).

    thzx
        Third rotation about local Y (positive Z toward X).

    par1
        Used for elliptical, spheroidal, or toroidal systems.  If KCS = 1
        or 2, PAR1 is the ratio of the ellipse Y-axis radius to X-axis
        radius (defaults to 1.0 (circle)).  If KCS = 3, PAR1 is the major
        radius of the torus.

    par2
        Used for spheroidal systems.  If KCS = 2, PAR2 = ratio of ellipse
        Z-axis radius to X-axis radius (defaults to 1.0 (circle)).

    Notes
    -----
    Defines and activates a local coordinate system by origin location and
    orientation angles relative to the active coordinate system.  This
    local system becomes the active coordinate system, and is automatically
    aligned with the active system (i.e., x is radial if a cylindrical
    system is active, etc.).  Nonzero rotation angles (degrees) are
    relative to this automatic rotation.  See the CS, CSKP, CSWPLA, and
    LOCAL commands for alternate definitions.  Local coordinate systems may
    be displayed with the /PSYMB command.

    This command is valid in any processor.
    """
    command = "CLOCAL,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(kcn), str(kcs), str(
        xl), str(yl), str(zl), str(thxy), str(thyz), str(thzx), str(par1), str(par2))
    return self.run(command, **kwargs)


def cs(self, kcn="", kcs="", norig="", nxax="", nxypl="", par1="", par2="",
       **kwargs):
    """Defines a local coordinate system by three node locations.

    APDL Command: CS

    Parameters
    ----------
    kcn
        Arbitrary reference number assigned to this coordinate system.
        Must be greater than 10.  A coordinate system previously defined
        with this number will be redefined.

    kcs
        Coordinate system type:

        0 or CART - Cartesian

        1 or CYLIN - Cylindrical (circular or elliptical)

        2 or SPHE - Spherical (or spheroidal)

        3 or TORO - Toroidal

    norig
        Node defining the origin of this coordinate system.  If NORIG = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).

    nxax
        Node defining the positive x-axis orientation of this coordinate
        system.

    nxypl
        Node defining the x-y plane (with NORIG and NXAX) in the first or
        second quadrant of this coordinate system.

    par1
        Used for elliptical, spheroidal, or toroidal systems.  If KCS = 1
        or 2, PAR1 is the ratio of the ellipse Y-axis radius to X-axis
        radius (defaults to 1.0 (circle)).  If KCS  = 3, PAR1 is the major
        radius of the torus.

    par2
        Used for spheroidal systems.  If KCS = 2, PAR2 = ratio of ellipse
        Z-axis radius to X-axis radius (defaults to 1.0 (circle)).

    Notes
    -----
    Defines and activates a local right-handed coordinate system by
    specifying three existing nodes: to locate the origin, to locate the
    positive x-axis, and to define the positive x-y plane.  This local
    system becomes the active coordinate system.  See the CLOCAL, CSKP,
    CSWPLA, and LOCAL commands for alternate definitions.  Local coordinate
    systems may be displayed with the /PSYMB command.

    This command is valid in any processor.
    """
    command = "CS,%s,%s,%s,%s,%s,%s,%s" % (str(kcn), str(kcs), str(
        norig), str(nxax), str(nxypl), str(par1), str(par2))
    return self.run(command, **kwargs)


def cscir(self, kcn="", kthet="", kphi="", **kwargs):
    """Locates the singularity for non-Cartesian local coordinate systems.

    APDL Command: CSCIR

    Parameters
    ----------
    kcn
        Number of the local coordinate system in which singularity location
        is to be changed.  Must be greater than 10.

    kthet
        Theta singularity location for cylindrical, spherical, and toroidal
        systems:

        0 - Singularity at ±180°.

        1 - Singularity at 0° (360°).

    kphi
        Phi singularity location for toroidal systems:

        0 - Singularity in phi direction at ±180°.

        1 - Singularity in phi direction at 0° (360°).

    Notes
    -----
    Continuous closed surfaces (circles, cylinders, spheres, etc.) have a
    singularity (discontinuity) at θ = ±180°. For local cylindrical,
    spherical, and toroidal coordinate systems, this singularity location
    may be changed to 0° (360°).

    An additional, similar singularity occurs in the toroidal coordinate
    system at: Φ = ±180° and can be moved with KPHI.  Additional
    singularities occur in the spherical coordinate system at: Φ = ±90°,
    but cannot be moved.

    This command is valid in any processor.
    """
    command = "CSCIR,%s,%s,%s" % (str(kcn), str(kthet), str(kphi))
    return self.run(command, **kwargs)


def csdele(self, kcn1="", kcn2="", kcinc="", **kwargs):
    """Deletes local coordinate systems.

    APDL Command: CSDELE

    Parameters
    ----------
    kcn1, kcn2, kcinc
        Delete coordinate systems from KCN1 (must be greater than 10) to
        KCN2 (defaults to KCN1) in steps of KCINC (defaults to 1).  If KCN1
        = ALL, KCN2 and KCINC are ignored and all coordinate systems are
        deleted.

    Notes
    -----
    This command is valid in any processor.
    """
    command = "CSDELE,%s,%s,%s" % (str(kcn1), str(kcn2), str(kcinc))
    return self.run(command, **kwargs)


def cskp(self, kcn="", kcs="", porig="", pxaxs="", pxypl="", par1="",
         par2="", **kwargs):
    """Defines a local coordinate system by three keypoint locations.

    APDL Command: CSKP

    Parameters
    ----------
    kcn
        Arbitrary reference number assigned to this coordinate system.
        Must be greater than 10.  A coordinate system previously defined
        with this number will be redefined.

    kcs
        Coordinate system type:

        0 or CART - Cartesian

        1 or CYLIN - Cylindrical (circular or elliptical)

        2 or SPHE - Spherical (or spheroidal)

        3 or TORO - Toroidal

    porig
        Keypoint defining the origin of this coordinate system.  If PORIG =
        P, graphical picking is enabled and all remaining command fields
        are ignored (valid only in the GUI).

    pxaxs
        Keypoint defining the positive x-axis orientation of this
        coordinate system.

    pxypl
        Keypoint defining the x-y plane (with PORIG and PXAXS) in the first
        or second quadrant of this coordinate system.

    par1
        Used for elliptical, spheroidal, or toroidal systems.  If KCS = 1
        or 2, PAR1 is the ratio of the ellipse Y-axis radius to X-axis
        radius (defaults to 1.0 (circle)).  If KCS = 3, PAR1 is the major
        radius of the torus.

    par2
        Used for spheroidal systems.  If KCS = 2, PAR2 = ratio of ellipse
        Z-axis radius to X-axis radius (defaults to 1.0 (circle)).

    Notes
    -----
    Defines and activates a local right-handed coordinate system by
    specifying three existing keypoints: to locate the origin, to locate
    the positive x-axis, and to define the positive x-y plane.  This local
    system becomes the active coordinate system.  See the CLOCAL, CS,
    CSWPLA, and LOCAL commands for alternate definitions.  Local coordinate
    systems may be displayed with the /PSYMB command.

    This command is valid in any processor.
    """
    command = "CSKP,%s,%s,%s,%s,%s,%s,%s" % (str(kcn), str(kcs), str(
        porig), str(pxaxs), str(pxypl), str(par1), str(par2))
    return self.run(command, **kwargs)


def cslist(self, kcn1="", kcn2="", kcinc="", **kwargs):
    """Lists coordinate systems.

    APDL Command: CSLIST

    Parameters
    ----------
    kcn1, kcn2, kcinc
        List coordinate systems from KCN1 to KCN2 (defaults to KCN1) in
        steps of KCINC (defaults to 1).  If KCN1 = ALL (default), KCN2 and
        KCINC are ignored and all coordinate systems are listed.

    Notes
    -----
    This command is valid in any processor.
    """
    command = "CSLIST,%s,%s,%s" % (str(kcn1), str(kcn2), str(kcinc))
    return self.run(command, **kwargs)


def cswpla(self, kcn="", kcs="", par1="", par2="", **kwargs):
    """Defines a local coordinate system at the origin of the working plane.

    APDL Command: CSWPLA

    Parameters
    ----------
    kcn
        Arbitrary reference number assigned to this coordinate system.
        Must be greater than 10.  A coordinate system previously defined
        with this number will be redefined.

    kcs
        Coordinate system type:

        0 or CART - Cartesian

        1 or CYLIN - Cylindrical (circular or elliptical)

        2 or SPHE - Spherical (or spheroidal)

        3 or TORO - Toroidal

    par1
        Used for elliptical, spheroidal, or toroidal systems.  If KCS = 1
        or 2, PAR1 is the ratio of the ellipse Y-axis radius to X-axis
        radius (defaults to 1.0 (circle)).  If KCS = 3, PAR1 is the major
        radius of the torus.

    par2
        Used for spheroidal systems.  If KCS = 2, PAR2 = ratio of ellipse
        Z-axis radius to X-axis radius (defaults to 1.0 (circle)).

    Notes
    -----
    Defines and activates a local right-handed coordinate system centered
    at the origin of the working plane.  The coordinate system's local x-y
    plane (for a Cartesian system) or R-θ plane (for a cylindrical or
    spherical system) corresponds to the working plane.  This local system
    becomes the active coordinate system.  See the CS, LOCAL, CLOCAL, and
    CSKP commands for alternate ways to define a local coordinate system.
    Local coordinate systems may be displayed with the /PSYMB command.

    This command is valid in any processor.
    """
    command = "CSWPLA,%s,%s,%s,%s" % (str(kcn), str(kcs), str(par1), str(par2))
    return self.run(command, **kwargs)


def csys(self, kcn="", **kwargs):
    """Activates a previously defined coordinate system.

    APDL Command: CSYS

    Parameters
    ----------
    kcn
        Specifies the active coordinate system, as follows:

        0 (default) - Cartesian

        1 - Cylindrical with global Cartesian Z as the axis of rotation

        2 - Spherical

        4 or WP - Working Plane

        5 - Cylindrical with global Cartesian Y as the axis of rotation

        11 or greater - Any previously defined local coordinate system

    Notes
    -----
    The CSYS command activates a previously defined coordinate system for
    geometry input and generation.  The LOCAL, CLOCAL, CS, CSKP, and CSWPLA
    commands also activate coordinate systems as they are defined. To set
    the active element coordinate system attribute pointer, issue the ESYS
    command.

    The active coordinate system for files created via the CDWRITE command
    is Cartesian (CSYS,0).

    This command is valid in any processor.

    CSYS,4 (or CSYS,WP) activates working plane tracking, which updates the
    coordinate system to follow working plane changes. To deactivate
    working plane tracking, activate any other coordinate system (for
    example, CSYS,0 or CSYS,11).

    CSYS,5 is a cylindrical coordinate system with global Cartesian Y as
    the axis. The local x, y and z axes are radial, θ, and axial
    (respectively). The R-Theta plane is the global X-Z plane, as it is for
    an axisymmetric model. Thus, at θ = 0.0, CSYS,5 has a specific
    orientation: the local x is in the global +X direction, local y is in
    the global -Z direction, and local z (the cylindrical axis) is in the
    global +Y direction.
    """
    command = "CSYS,%s" % (str(kcn))
    return self.run(command, **kwargs)


def local(self, kcn="", kcs="", xc="", yc="", zc="", thxy="", thyz="",
          thzx="", par1="", par2="", **kwargs):
    """Defines a local coordinate system by a location and orientation.

    APDL Command: LOCAL

    Parameters
    ----------
    kcn
        Arbitrary reference number assigned to this coordinate system.
        Must be greater than 10.  A coordinate system previously defined
        with this number will be redefined.

    kcs
        Coordinate system type:

        0 or CART - Cartesian

        1 or CYLIN - Cylindrical (circular or elliptical)

        2 or SPHE - Spherical (or spheroidal)

        3 or TORO - Toroidal

    xc, yc, zc
        Location (in the global Cartesian coordinate system) of the origin
        of the new coordinate system.

    thxy
        First rotation about local Z (positive X toward Y).

    thyz
        Second rotation about local X (positive Y toward Z).

    thzx
        Third rotation about local Y (positive Z toward X).

    par1
        Used for elliptical, spheroidal, or toroidal systems.  If KCS = 1
        or 2, PAR1 is the ratio of the ellipse Y-axis radius to X-axis
        radius (defaults to 1.0 (circle)).  If KCS = 3, PAR1 is the major
        radius of the torus.

    par2
        Used for spheroidal systems.  If KCS = 2, PAR2 = ratio of ellipse
        Z-axis radius to X-axis radius (defaults to 1.0 (circle)).

    Notes
    -----
    Defines a local coordinate system by origin location and orientation
    angles.  The local coordinate system is parallel to the global
    Cartesian system unless rotated.  Rotation angles are in degrees and
    redefine any previous rotation angles.  See the CLOCAL, CS, CSWPLA, and
    CSKP commands for alternate definitions.  This local system becomes the
    active coordinate system [CSYS].  Local coordinate systems may be
    displayed with the /PSYMB command.

    This command is valid in any processor.
    """
    command = "LOCAL,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(kcn), str(kcs), str(
        xc), str(yc), str(zc), str(thxy), str(thyz), str(thzx), str(par1), str(par2))
    return self.run(command, **kwargs)



from ansys.mapdl.core._commands import parse


def blc4(self, xcorner="", ycorner="", width="", height="", depth="",
         **kwargs) -> int:
    """Creates a rectangular area or block volume by corner points.

    APDL Command: BLC4

    Parameters
    ----------
    xcorner, ycorner
        Working plane X and Y coordinates of one corner of the
        rectangle or block face.

    width
        The distance from XCORNER on or parallel to the working
        plane X-axis that, together with YCORNER, defines a second
        corner of the rectangle or block face.

    height
        The distance from YCORNER on or parallel to the working
        plane Y-axis that, together with XCORNER, defines a third
        corner of the rectangle or block face.

    depth
        The perpendicular distance (either positive or negative
        based on the working plane Z direction) from the working
        plane representing the depth of the block.  If DEPTH = 0
        (default), a rectangular area is created on the working
        plane.

    Returns
    -------
    int
        Volume or area number of the block or rectangle.

    Examples
    --------
    Create a block with dimensions 1 x 2 x 10 with one corner of
    the block at (0, 0) of the current working plane.

    >>> vnum = mapdl.blc4(1, 1, 1, 2, 10)
    >>> vnum
    1

    Notes
    -----
    Defines a rectangular area anywhere on the working plane or a
    hexahedral volume with one face anywhere on the working plane.
    A rectangle will be defined with four keypoints and four
    lines.  A volume will be defined with eight keypoints, twelve
    lines, and six areas, with the top and bottom faces parallel
    to the working plane.  See the BLC5, RECTNG, and BLOCK
    commands for alternate ways to create rectangles and blocks.
    """
    command = f"BLC4,{xcorner},{ycorner},{width},{height},{depth}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def blc5(self, xcenter="", ycenter="", width="", height="", depth="",
         **kwargs) -> int:
    """Create a rectangular area or block volume by center and corner

    APDL Command: BLC5
    points.

    Parameters
    ----------
    xcenter, ycenter
        Working plane X and Y coordinates of the center of the
        rectangle or block face.

    width
        The total distance on or parallel to the working plane
        X-axis defining the width of the rectangle or block face.

    height
        The total distance on or parallel to the working plane
        Y-axis defining the height of the rectangle or block face.

    depth
        The perpendicular distance (either positive or negative
        based on the working plane Z direction) from the working
        plane representing the depth of the block.  If ``depth=0``
        (default), a rectangular area is created on the working
        plane.

    Returns
    -------
    int
        Volume or area number of the block or rectangle.

    Examples
    --------
    Create a square centered at ``(0, 0)`` with a width of 0.5 and
    a height of 0.5

    >>> anum = mapdl.blc5(width=0.5, height=0.5)
    >>> anum
    1

    >>> vnum = mapdl.blc5(width=1, height=4, depth=9)
    >>> vnum
    1

    Notes
    -----
    Defines a rectangular area anywhere on the working plane or a
    hexahedral volume with one face anywhere on the working plane
    by specifying the center and corner points.  A rectangle will
    be defined with four keypoints and four lines.  A volume will
    be defined with eight keypoints, twelve lines, and six areas,
    with the top and bottom faces parallel to the working plane.
    See the ``BLC4``, ``RECTNG``, and ``BLOCK`` commands for
    alternate ways to create rectangles and blocks.
    """
    command = f"BLC5,{xcenter},{ycenter},{width},{height},{depth}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def block(self, x1="", x2="", y1="", y2="", z1="", z2="", **kwargs):
    """Create a block volume based on working plane coordinates.

    APDL Command: BLOCK

    Parameters
    ----------
    x1, x2
        Working plane X coordinates of the block.

    y1, y2
        Working plane Y coordinates of the block.

    z1, z2
        Working plane Z coordinates of the block.

    Returns
    -------
    int
        Volume number of the block.

    Examples
    --------
    Create a block volume based on working plane coordinates with
    the size ``(1 x 2 x 3)``.

    >>> vnum = mapdl.block(0, 1, 0, 2, 1, 4)
    >>> vnum
    1

    Notes
    -----
    Defines a hexahedral volume based on the working plane.  The
    block must have a spatial volume greater than zero (i.e., this
    volume primitive command cannot be used to create a degenerate
    volume as a means of creating an area.)  The volume will be
    defined with eight keypoints, twelve lines, and six areas,
    with the top and bottom faces parallel to the working plane.
    See the ``BLC4`` and ``BLC5`` commands for alternate ways to
    create blocks.
    """
    command = f"BLOCK,{x1},{x2},{y1},{y2},{z1},{z2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def con4(self, xcenter="", ycenter="", rad1="", rad2="", depth="",
         **kwargs) -> int:
    """Create a conical volume anywhere on the working plane.

    APDL Command: CON4

    Parameters
    ----------
    xcenter, ycenter
        Working plane X and Y coordinates of the center axis of
        the cone.

    rad1, rad2
        Radii of the faces of the cone.  RAD1 defines the bottom
        face and will be located on the working plane.  RAD2
        defines the top face and is parallel to the working plane.
        A value of zero or blank for either RAD1 or RAD2 defines a
        degenerate face at the center axis (i.e., the vertex of
        the cone).  The same value for both RAD1 and RAD2 defines
        a cylinder instead of a cone.

    depth
        The perpendicular distance (either positive or negative
        based on the working plane Z direction) from the working
        plane representing the depth of the cone.  DEPTH cannot be
        zero (see "Notes" below).

    Returns
    -------
    int
        Volume number of the cone.

    Examples
    --------
    Create a cone with a bottom radius of 3 and a height of 10.

    >>> vnum = mapdl.con4(rad1=3, rad2=0, depth=10)
    >>> vnum
    1

    Notes
    -----
    Defines a solid conical volume with either the vertex or a
    face anywhere on the working plane.  The cone must have a
    spatial volume greater than zero.  (i.e., this volume
    primitive command cannot be used to create a degenerate volume
    as a means of creating an area.)  The face or faces will be
    circular (each area defined with four lines), and they will be
    connected with two areas (each spanning 180 degrees).  See the CONE
    command for an alternate way to create cones.
    """
    command = f"CON4,{xcenter},{ycenter},{rad1},{rad2},{depth}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def cone(self, rbot="", rtop="", z1="", z2="", theta1="", theta2="",
         **kwargs) -> int:
    """Create a conical volume centered about the working plane origin.

    APDL Command: CONE

    Parameters
    ----------
    rbot, rtop
        Radii of the bottom and top faces of the cone.  A value of
        zero or blank for either RBOT or RTOP defines a degenerate
        face at the center axis (i.e., the vertex of the cone).
        The same value for both RBOT and RTOP defines a cylinder
        instead of a cone.

    z1, z2
        Working plane Z coordinates of the cone.  The smaller
        value is always associated with the bottom face.

    theta1, theta2
        Starting and ending angles (either order) of the cone.
        Used for creating a conical sector.  The sector begins at
        the algebraically smaller angle, extends in a positive
        angular direction, and ends at the larger angle.  The
        starting angle defaults to 0 degrees and the ending angle
        defaults to 360 degrees.  See the Modeling and Meshing Guide for
        an illustration.

    Returns
    -------
    int
        Volume number of the cone.

    Examples
    --------
    Create a quarter cone with a bottom radius of 3, top radius of 1 and
    a height of 10 centered at ``(0, 0)``.

    >>> vnum = mapdl.cone(rbot=5, rtop=1, z1=0, z2=10, theta1=180, theta2=90)
    >>> vnum
    1

    Notes
    -----
    Defines a solid conical volume centered about the working
    plane origin.  The non-degenerate face (top or bottom) is
    parallel to the working plane but not necessarily coplanar
    with (i.e., "on") the working plane.  The cone must have a
    spatial volume greater than zero. (i.e., this volume primitive
    command cannot be used to create a degenerate volume as a
    means of creating an area.)

    For a cone of 360, top and bottom faces will be circular (each
    area defined with four lines), and they will be connected with
    two areas (each spanning 180 degrees).  See the ``CON4``
    command for an alternate way to create cones.
    """
    command = f"CONE,{rbot},{rtop},{z1},{z2},{theta1},{theta2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def cyl4(self, xcenter="", ycenter="", rad1="", theta1="", rad2="",
         theta2="", depth="", **kwargs) -> int:
    """Creates a circular area or cylindrical volume anywhere on
    the working plane.

    APDL Command: CYL4

    Parameters
    ----------
    xcenter, ycenter
        Working plane X and Y coordinates of the center of the
        circle or cylinder.

    rad1, rad2
        Inner and outer radii (either order) of the circle or
        cylinder.  A value of zero or blank for either RAD1 or
        RAD2, or the same value for both RAD1 and RAD2, defines a
        solid circle or cylinder.

    theta1, theta2
        Starting and ending angles (either order) of the circle or
        faces of the cylinder.  Used for creating a partial
        annulus or partial cylinder.  The sector begins at the
        algebraically smaller angle, extends in a positive angular
        direction, and ends at the larger angle.  The starting
        angle defaults to 0 degrees and the ending angle defaults to
        360 degrees.  See the Modeling and Meshing Guide for an
        illustration.

    depth
        The perpendicular distance (either positive or negative
        based on the working plane Z direction) from the working
        plane representing the depth of the cylinder.  If DEPTH =
        0 (default), a circular area is created on the working
        plane.

    Returns
    -------
    int
        Volume or area number of the block or rectangle.

    Examples
    --------
    Create a half arc centered at the origin with an outer radius
    of 2 and an inner radius of 1

    >>> anum = mapdl.cyl4(xcenter=0, ycenter=0, rad1=1,
                          theta1=0, rad2=2, theta2=180)
    >>> anum

    Create a solid cylinder with a depth of 10 at the center of
    the working plane.

    >>> vnum = mapdl.cyl4(0, 0, 1, depth=10)
    >>> vnum
    1

    Create a cylinder with an inner radius of 1.9 and an outer of
    2.0 with a height of 5 centered at the working plane.

    >>> vnum = mapdl.cyl4(0, 0, rad1=1.9, rad2=2.0, depth=10)
    2

    Notes
    -----
    Defines a circular area anywhere on the working plane or a
    cylindrical volume with one face anywhere on the working
    plane.  For a solid cylinder of 360 degrees, the top and bottom faces
    will be circular (each area defined with four lines) and they
    will be connected with two surface areas (each spanning 180 degrees).
    See the CYL5, PCIRC, and CYLIND commands for alternate ways to
    create circles and cylinders.

    When working with a model imported from an IGES file (DEFAULT
    import option), you must provide a value for DEPTH or the
    command will be ignored.
    """
    command = f"CYL4,{xcenter},{ycenter},{rad1},{theta1},{rad2},{theta2},{depth}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def cyl5(self, xedge1="", yedge1="", xedge2="", yedge2="", depth="",
         **kwargs) -> int:
    """Create a circular area or cylindrical volume by end points.

    APDL Command: CYL5

    Parameters
    ----------
    xedge1, yedge1
        Working plane X and Y coordinates of one end of the circle
        or cylinder face.

    xedge2, yedge2
        Working plane X and Y coordinates of the other end of the
        circle or cylinder face.

    depth
        The perpendicular distance (either positive or negative
        based on the working plane Z direction) from the working
        plane representing the depth of the cylinder.  If DEPTH =
        0 (default), a circular area is created on the working
        plane.

    Returns
    -------
    int
        Volume or area number of the circular area of cylindrical
        volume.


    Examples
    --------
    Create a circular with one point of the circle at ``(1, 1)``
    and the other point at ``(2, 2)``

    >>> anum = mapdl.cyl5(xedge1=1, yedge1=1, xedge2=2, yedge2=2)
    >>> anum
    1

    Create a cylinder with one point of the circle at ``(X, Y) ==
    (1, 1)`` and the other point at ``(X, Y) == (2, 2)`` with a
    height of 3.

    >>> vnum = mapdl.cyl5(xedge1=1, yedge1=1, xedge2=2, yedge2=2, depth=5)
    >>> vnum
    1

    Notes
    -----
    Defines a circular area anywhere on the working plane or a
    cylindrical volume with one face anywhere on the working plane
    by specifying diameter end points.  For a solid cylinder of
    360°, the top and bottom faces will be circular (each area
    defined with four lines) and they will be connected with two
    surface areas (each spanning 180°).  See the CYL4, PCIRC, and
    CYLIND commands for alternate ways to create circles and
    cylinders.
    """
    command = f"CYL5,{xedge1},{yedge1},{xedge2},{yedge2},{depth}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def cylind(self, rad1="", rad2="", z1="", z2="", theta1="", theta2="",
           **kwargs) -> int:
    """Create a cylindrical volume centered about the working plane origin.

    APDL Command: CYLIND

    Parameters
    ----------
    rad1, rad2
        Inner and outer radii (either order) of the cylinder.  A
        value of zero or blank for either RAD1 or RAD2, or the
        same value for both RAD1 and RAD2, defines a solid
        cylinder.

    z1, z2
        Working plane Z coordinates of the cylinder.  If either Z1
        or Z2 is zero, one of the faces of the cylinder will be
        coplanar with the working plane.

    theta1, theta2
        Starting and ending angles (either order) of the cylinder.
        Used for creating a cylindrical sector.  The sector begins
        at the algebraically smaller angle, extends in a positive
        angular direction, and ends at the larger angle.  The
        starting angle defaults to 0.0° and the ending angle
        defaults to 360.0°.  See the Modeling and Meshing Guide
        for an illustration.

    Returns
    -------
    int
        Volume number of the cylinder.

    Examples
    --------
    Create a hollow cylinder with an inner radius of 0.9 and an
    outer radius of 1.0 with a height of 5

    >>> vnum = mapdl.cylind(0.9, 1, z1=0, z2=5)
    >>> vnum
    1

    Notes
    -----
    Defines a cylindrical volume centered about the working plane
    origin.  The top and bottom faces are parallel to the working
    plane but neither face need be coplanar with (i.e., "on") the
    working plane.  The cylinder must have a spatial volume
    greater than zero. (i.e., this volume primitive command cannot
    be used to create a degenerate volume as a means of creating
    an area.)

    For a solid cylinder of 360°, the top and bottom faces will be
    circular (each area defined with four lines), and they will be
    connected with two areas (each spanning 180°.)  See the CYL4
    and CYL5 commands for alternate ways to create cylinders.
    """
    command = f"CYLIND,{rad1},{rad2},{z1},{z2},{theta1},{theta2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def pcirc(self, rad1="", rad2="", theta1="", theta2="", **kwargs) -> int:
    """Create a circular area centered about the working plane origin.

    APDL Command: PCIRC

    Parameters
    ----------
    rad1, rad2
        Inner and outer radii (either order) of the circle.  A
        value of either zero or blank for either ``rad1`` or
        ``rad2``, or the same value for both ``rad1`` and
        ``rad2``, defines a solid circle.

    theta1, theta2
        Starting and ending angles (either order) of the circular
        area.  Used for creating a circular sector.  The sector
        begins at the algebraically smaller angle, extends in a
        positive angular direction, and ends at the larger angle.
        The starting angle defaults to 0.0° and the ending angle
        defaults to 360.0°.  See the Modeling and Meshing Guide
        for an illustration.

    Returns
    -------
    int
        Area number of the new circular area.

    Examples
    --------
    In this example a circular area with an inner radius of 0.95
    and an outer radius of 1 is created.

    >>> anum = mapdl.pcirc(0.95, 1)
    >>> anum
    1

    Notes
    -----
    Defines a solid circular area or circular sector centered
    about the working plane origin.  For a solid circle of 360°,
    the area will be defined with four keypoints and four lines.
    See the ``cyl4`` and ``cyl5`` commands for alternate ways to
    create circles.
    """
    command = f"PCIRC,{rad1},{rad2},{theta1},{theta2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def poly(self, **kwargs):
    """Creates a polygonal area based on working plane coordinate pairs.

    APDL Command: POLY

    Notes
    -----
    Defines a polygonal area on the working plane.  The area will be
    defined with NPT keypoints and NPT lines, where NPT (must be at least
    3) is the number of coordinate pairs defined with the PTXY command.
    See the RPOLY and RPR4 commands for other ways to create polygons.
    """
    command = f"POLY,"
    return self.run(command, **kwargs)


def pri2(self, p51x="", z1="", z2="", **kwargs):
    """Creates a polygonal area or a prism volume by vertices (GUI).

    APDL Command: PRI2

    Notes
    -----
    Creates a polygonal area or a prism volume using the vertices as input.
    This is a command generated by the Graphical User Interface (GUI) and
    will appear in the log file (Jobname.LOG) if graphical picking is used.
    This command is not intended to be typed in directly in an ANSYS
    session (although it can be included in an input file for batch input
    or for use with the /INPUT command).

    For polygons, the PRI2 command will appear in the log file as
    PRI2,P51X,0.0,0.0 preceded by FITEM commands that define the vertices
    (in global Cartesian coordinates).  For prisms, the PRI2 command will
    appear in the log file as PRI2,P51X preceded by FITEM commands that
    define the vertices and the Z-end of the prism.

    See the RPOLY, POLY, RPRISM, PRISM, and RPR4 commands for other ways to
    create polygons and prisms.
    """
    command = f"PRI2,{p51x},{z1},{z2}"
    return self.run(command, **kwargs)


def prism(self, z1="", z2="", **kwargs):
    """Creates a prism volume based on working plane coordinate pairs.

    APDL Command: PRISM

    Parameters
    ----------
    z1, z2
        Working plane Z coordinates of the top and bottom of the prism.

    Notes
    -----
    Defines a prism volume based on the working plane. The top and bottom
    areas will each be defined with NPT keypoints and NPT lines, where NPT
    (must be at least 3) is the number of coordinate pairs defined with
    PTXY command.  Also, a line will be defined between each point pair on
    the top and bottom face.  See the RPRISM and RPR4 commands for other
    ways to create prisms.
    """
    command = f"PRISM,{z1},{z2}"
    return self.run(command, **kwargs)


def ptxy(self, x1="", y1="", x2="", y2="", x3="", y3="", x4="", y4="",
         **kwargs):
    """Defines coordinate pairs for use in polygons and prisms.

    APDL Command: PTXY

    Parameters
    ----------
    x1, y1, x2, y2, x3, y3, x4, y4
        X and Y coordinate pairs on the working plane.

    Notes
    -----
    Defines coordinate pairs for use in polygons and prisms [POLY, RPRISM].
    The coordinates must be in the Cartesian coordinate system.  The
    coordinate pairs must be input in a continuous order.  PTXY  may be
    repeated (up to 100 pairs) until the required pairs have been defined.
    The pairs will be saved until either the POLY or PRISM command is
    entered.  Use PTXY,STAT to list the saved coordinate pairs.  Use
    PTXY,DELE to delete all the saved coordinate pairs.  See the RPOLY,
    RPRISM, and RPR4 commands for other ways to create polygons and prisms.
    """
    command = f"PTXY,{x1},{y1},{x2},{y2},{x3},{y3},{x4},{y4}"
    return self.run(command, **kwargs)


def rectng(self, x1="", x2="", y1="", y2="", **kwargs):
    """Create a rectangular area anywhere on the working plane.

    APDL Command: RECTNG

    Parameters
    ----------
    x1, x2
        Working plane X coordinates of the rectangle.

    y1, y2
        Working plane Y coordinates of the rectangle.

    Notes
    -----
    The area will be defined with four keypoints and four lines.
    See the ``blc4`` and ``blc5`` commands for alternate ways to
    create rectangles.
    """
    command = f"RECTNG,{x1},{x2},{y1},{y2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def rpoly(self, nsides="", lside="", majrad="", minrad="", **kwargs):
    """Creates a regular polygonal area centered about the working plane

    APDL Command: RPOLY
    origin.

    Parameters
    ----------
    nsides
        Number of sides in the regular polygon.  Must be greater than 2.

    lside
        Length of each side of the regular polygon.

    majrad
        Radius of the major (or circumscribed) circle of the polygon.  Not
        used if LSIDE is input.

    minrad
        Radius of the minor (or inscribed) circle of the polygon.  Not used
        if LSIDE or MAJRAD is input.

    Notes
    -----
    Defines a regular polygonal area on the working plane.  The polygon
    will be centered about the working plane origin, with the first
    keypoint defined at : θ = 0°.  The area will be defined with NSIDES
    keypoints and NSIDES lines.  See the RPR4 and POLY commands for other
    ways to create polygons.
    """
    command = f"RPOLY,{nsides},{lside},{majrad},{minrad}"
    return self.run(command, **kwargs)


def rpr4(self, nsides="", xcenter="", ycenter="", radius="", theta="",
         depth="", **kwargs):
    """Creates a regular polygonal area or prism volume anywhere on the

    APDL Command: RPR4
    working plane.

    Parameters
    ----------
    nsides
        The number of sides in the polygon or prism face.  Must be greater
        than 2.

    xcenter, ycenter
        Working plane X and Y coordinates of the center of the polygon or
        prism face.

    radius
        Distance (major radius) from the center to a vertex of the polygon
        or prism face (where the first keypoint is defined).

    theta
        Angle (in degrees) from the working plane X-axis to the vertex of
        the polygon or prism face where the first keypoint is defined.
        Used to orient the polygon or prism face.  Defaults to zero.

    depth
        The perpendicular distance (either positive or negative based on
        the working plane Z direction) from the working plane representing
        the depth of the prism.  If DEPTH = 0 (default), a polygonal area
        is created on the working plane.

    Notes
    -----
    Defines a regular polygonal area anywhere on the working plane or prism
    volume with one face anywhere on the working plane.  The top and bottom
    faces of the prism are polygonal areas.  See the RPOLY, POLY, RPRISM,
    and PRISM commands for other ways to create polygons and prisms.
    """
    command = f"RPR4,{nsides},{xcenter},{ycenter},{radius},{theta},{depth}"
    return self.run(command, **kwargs)


def rprism(self, z1="", z2="", nsides="", lside="", majrad="", minrad="",
           **kwargs):
    """Creates a regular prism volume centered about the working plane origin.

    APDL Command: RPRISM

    Parameters
    ----------
    z1, z2
        Working plane Z coordinates of the prism.

    nsides
        Number of sides in the polygon defining the top and bottom faces of
        the prism.  Must be greater than 2.

    lside
        Length of each side of the polygon defining the top and bottom
        faces of the prism.

    majrad
        Radius of the major (or circumscribed) circle of the polygon
        defining the top and bottom faces of the prism.  Not used if LSIDE
        is input.

    minrad
        Radius of the minor (or inscribed circle) of the polygon defining
        the top and bottom faces of the prism.  Not used if LSIDE or MAJRAD
        is input.

    Notes
    -----
    Defines a regular prism volume centered about the working plane origin.
    The prism must have a spatial volume greater than zero. (i.e., this
    volume primitive command cannot be used to create a degenerate volume
    as a means of creating an area.)  The top and bottom faces are
    polygonal areas that are parallel to the working plane but neither face
    need be coplanar with (i.e., "on") the working plane. The first
    keypoint defined for each face is at : θ = 0°.  See the RPR4 and PRISM
    commands for other ways to create prisms.
    """
    command = f"RPRISM,{z1},{z2},{nsides},{lside},{majrad},{minrad}"
    return self.run(command, **kwargs)


def sph4(self, xcenter="", ycenter="", rad1="", rad2="", **kwargs) -> int:
    """Create a spherical volume anywhere on the working plane.

    APDL Command: SPH4

    Parameters
    ----------
    xcenter, ycenter
        Working plane X and Y coordinates of the center of the
        sphere.

    rad1, rad2
        Inner and outer radii (either order) of the sphere.  A
        value of zero or blank for either ``rad1`` or ``rad2``
        defines a solid sphere.

    Returns
    -------
    int
        Volume number of the sphere.


    Examples
    --------
    This example creates a hollow sphere with an inner radius of
    0.9 and an outer radius of 1.0 centered at ``(0, 0)``

    >>> vnum = mapdl.sph4(0, 0, rad1=0.9, rad2=1.0)
    >>> vnum
    1

    Notes
    -----
    Defines either a solid or hollow spherical volume anywhere on
    the working plane.  The sphere must have a spatial volume
    greater than zero.  (i.e., this volume primitive command
    cannot be used to create a degenerate volume as a means of
    creating an area.)  A sphere of 360° will be defined with two
    areas, each consisting of a hemisphere.  See the ``sphere``
    and ``sph5`` commands for other ways to create spheres.

    When working with a model imported from an IGES file (DEFAULT
    import option), you can create only solid spheres.  If you
    enter a value for both ``rad1`` and ``rad2`` the command is
    ignored.
    """
    command = f"SPH4,{xcenter},{ycenter},{rad1},{rad2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def sph5(self, xedge1="", yedge1="", xedge2="", yedge2="", **kwargs) -> int:
    """Create a spherical volume by diameter end points.

    APDL Command: SPH5

    Parameters
    ----------
    xedge1, yedge1
        Working plane X and Y coordinates of one edge of the sphere.

    xedge2, yedge2
        Working plane X and Y coordinates of the other edge of the
        sphere.

    Returns
    -------
    int
        Volume number of the sphere.

    Examples
    --------
    This example creates a sphere with one point at ``(1, 1)`` and
    one point at ``(2, 2)``

    >>> vnum = mapdl.sph5(xedge1=1, yedge1=1, xedge2=2, yedge2=2)
    >>> vnum
    1

    Notes
    -----
    Defines a solid spherical volume anywhere on the working plane
    by specifying diameter end points.  The sphere must have a
    spatial volume greater than zero.  (i.e., this volume
    primitive command cannot be used to create a degenerate volume
    as a means of creating an area.)  A sphere of 360° will be
    defined with two areas, each consisting of a hemisphere.  See
    the ``sphere`` and ``sph4`` commands for other ways to create
    spheres.
    """
    command = f"SPH5,{xedge1},{yedge1},{xedge2},{yedge2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def sphere(self, rad1="", rad2="", theta1="", theta2="", **kwargs) -> int:
    """Create a spherical volume centered about the working plane origin.

    APDL Command: SPHERE

    Parameters
    ----------
    rad1, rad2
        Inner and outer radii (either order) of the sphere.  A
        value of zero or blank for either ``rad1`` or ``rad2``
        defines a solid sphere.

    theta1, theta2
        Starting and ending angles (either order) of the sphere.
        Used for creating a spherical sector.  The sector begins
        at the algebraically smaller angle, extends in a positive
        angular direction, and ends at the larger angle.  The
        starting angle defaults to 0.0° and the ending angle
        defaults to 360.0°.  See the Modeling and Meshing Guide
        for an illustration.

    Returns
    -------
    int
        Volume number of the sphere.

    Examples
    --------
    >>> vnum = mapdl.sphere(rad1=0.95, rad2=1.0, theta1=90, theta2=270)
    >>> vnum
    1

    Notes
    -----
    Defines either a solid or hollow sphere or spherical sector
    centered about the working plane origin.  The sphere must have
    a spatial volume greater than zero. (i.e., this volume
    primitive command cannot be used to create a degenerate volume
    as a means of creating an area.)  Inaccuracies can develop
    when the size of the object you create is much smaller than
    the relative coordinate system values (ratios near to or
    greater than 1000). If you require an exceptionally small
    sphere, create a larger object, and scale it down to the
    appropriate size.

    For a solid sphere of 360°, you define it with two areas, each
    consisting of a hemisphere.  See the ``sph4`` and ``sph5``
    commands for the other ways to create spheres.
    """
    command = f"SPHERE,{rad1},{rad2},{theta1},{theta2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))


def torus(self, rad1="", rad2="", rad3="", theta1="", theta2="", **kwargs):
    """Create a toroidal volume.

    APDL Command: TORUS

    Parameters
    ----------
    rad1, rad2, rad3
        Three values that define the radii of the torus.  You can
        specify the radii in any order.  The smallest of the
        values is the inner minor radius, the intermediate value
        is the outer minor radius, and the largest value is the
        major radius.  (There is one exception regarding the order
        of the radii values--if you want to create a solid torus,
        specify zero or blank for the inner minor radius, in which
        case the zero or blank must occupy either the ``rad1`` or
        ``rad2`` position.)

        At least two of the values that you specify must be
        positive values; they will be used to define the outer
        minor radius and the major radius.  See the diagram in the
        Notes section for a view of a toroidal sector showing all
        radii.

    theta1, theta2
        Starting and ending angles (either order) of the torus.
        Used for creating a toroidal sector.  The sector begins at
        the algebraically smaller angle, extends in a positive
        angular direction, and ends at the larger angle.  The
        starting angle defaults to 0° and the ending angle
        defaults to 360°.

    Returns
    -------
    int
        Volume number of the torus.


    Examples
    --------
    This example creates a torus with an inner minor radius of 1, an
    intermediate radii of 2, and a major radius of 5.  The values
    0 and 180 define the starting and ending angles of the torus.

    >>> vnum = mapdl.torus(rad1=5, rad2=1, rad3=2, theta1=0, theta2=180)
    >>> vnum
    1

    Notes
    -----
    Defines a toroidal volume centered about the working plane
    origin.  A solid torus of 360° will be defined with four
    areas, each area spanning 180° around the major and minor
    circumference.
    """
    command = f"TORUS,{rad1},{rad2},{rad3},{theta1},{theta2}"
    return parse.parse_output_volume_area(self.run(command, **kwargs))



def an3d(self, kywrd="", key="", **kwargs):
    """Specifies 3-D annotation functions

    APDL Command: /AN3D

    Parameters
    ----------
    num
        Unique number assigned as each annotation is applied to a model.
        These numbers are applied sequentially, although when an annotation
        entity is deleted, its number is reassigned.

    type
        Annotation internal type number (101 = text, 102 = line, 103 =
        point, 104 = area, 105 = arrow, 106 = symbol, 108 = bitmap).

    xhot, yhot, zhot
        X, Y, Z coordinates for hot spot location.

    Notes
    -----
    Because 3-D annotation is applied in relation to the XYZ coordinates of
    the anchor, you can transform your model, and the annotation will
    maintain the spatial relationship with the model. This works within
    reason, and there are instances where changing the perspective or the
    size of the model will change the apparent relationship between the
    annotation and the model.

     The overall 3-D dimensions of your model are defined by a bounding
    box.  If portions of your model's bounding box lie outside of the
    visible area of your graphics window (if you are zoomed in on a
    specific area of your model), it can affect the placement of your 3-D
    annotations.  Zooming out will usually overcome this problem.

    3-D annotation is valid for the Cartesian (CSYS,0) coordinate system
    only. If you want to annotate a model you created in another coordinate
    system, use 2-D annotation (note that 2-D annotations do not remain
    anchored for dynamic rotations or transformations).

    When you apply user defined bitmaps, the size of the annotation can
    vary. Use the options menu of the 3-D annotation widget to adjust the
    size and placement of your bitmaps.

    You cannot use the "!" and "$" characters in ANSYS text annotation.

    The GUI generates this command during 3-D annotation operations and
    inserts the command into the log file (Jobname.LOG). You should NOT
    type this command directly during an ANSYS session (although the
    command can be included in an input file for batch input or for use
    with the /INPUT command).
    """
    command = f"/AN3D,{kywrd},{key}"
    return self.run(command, **kwargs)


def annot(self, lab="", val1="", val2="", **kwargs):
    """Activates graphics for annotating displays (GUI).

    APDL Command: /ANNOT

    Parameters
    ----------
    lab
        Annotation control key:

        OFF - Turns off annotation for each subsequent display (default).

        ON - Turns on annotation for each subsequent display.

        DELE - Deletes all annotation.

        SAVE - Saves annotation on a file.  Use VAL1 for file name (defaults to Jobname) and
               VAL2 for the extension (defaults to ANO).

        SCALE - Sets annotation scale factor (direct input only).  Use VAL1 for value (0.1 to
                10.0) (defaults to 1.0).

        XORIG - Sets the annotation x origin (direct input only).  Use VAL1 for value (-3.0 to
                3.0).

        YORIG - Sets annotation y origin (direct input only).  Use VAL1 for value (-3.0 to
                3.0).

        SNAP - Sets annotation snap (menu button input only).  Use VAL1 for value (0.002 to
               0.2) (defaults to 0.002).

        STAT - Displays current annotation status.

        DEFA - Sets annotation specifications to the default values.

        REFR - Redisplays annotation graphics.

        TMOD - Sets the annotation text mode. If VAL1 = 1, annotation text will be drawn in
               scalable bitmap fonts (default). If VAL1 = 0, annotation
               text will be drawn with stroke text.

    val1
        Value (or file name) as noted with label above.

    val2
        Value (or file name extension) as noted with label above.

    Notes
    -----
    This is a command generated by the GUI and will appear in the log file
    (Jobname.LOG) if annotation is used.  This command is not intended to
    be typed in directly in an ANSYS session (although it can be included
    in an input file for batch input or for use with the /INPUT command).

    You cannot use the "!" and "$" characters in ANSYS text annotation.

    /ANNOT activates annotation graphics for adding annotation to displays.
    Commands representing the annotation instructions are automatically
    created by the annotation functions in the GUI and written to
    Jobname.LOG.  The annotation commands are /ANNOT, /ANUM, /TLABEL,
    /LINE, /LARC, /LSYMBOL, /POLYGON, /PMORE, /PCIRCLE, /PWEDGE, /TSPEC,
    /LSPEC, and /PSPEC.  Annotation graphics are relative to the full
    Graphics Window and are not affected by ANSYS window-specific commands
    (/WINDOW, /VIEW, etc.).

    This command is valid in any processor.
    """
    command = f"/ANNOT,{lab},{val1},{val2}"
    return self.run(command, **kwargs)


def anum(self, num="", type_="", xhot="", yhot="", **kwargs):
    """Specifies the annotation number, type, and hot spot (GUI).

    APDL Command: /ANUM

    Parameters
    ----------
    num
        Annotation number.  ANSYS automatically assigns the lowest
        available number.  You cannot assign a higher number if a lower
        number is available; ANSYS will substitute the lowest available
        number in place of any user-specified higher number.

    type\_
        Annotation internal type number.  If TYPE = DELE, delete annotation
        NUM.

        1 - Text

        2 - Block text (not available in GUI)

        3 - Dimensions

        4 - Lines

        5 - Rectangles

        6 - Circles

        7 - Polygons

        8 - Arcs

        9 - Wedges, pies

        11 - Symbols

        12 - Arrows

        13 - Bitmap

    xhot
        X hot spot (-1.0 < X < 2.0).  Used for menu button item delete.

    yhot
        Y hot spot (-1.0 < Y < 1.0).  Used for menu button item delete.

    Notes
    -----
    This is a command generated by the GUI and will appear in the log file
    (Jobname.LOG) if annotation is used.  This command is not intended to
    be typed in directly in an ANSYS session (although it can be included
    in an input file for batch input or for use with the /INPUT command).

    Type 13 (bitmap) annotation applies user defined bitmaps defined using
    the FILE option of the /TXTRE command.

    This command is valid in any processor.
    """
    command = f"/ANUM,{num},{type_},{xhot},{yhot}"
    return self.run(command, **kwargs)


def lspec(self, lcolor="", linstl="", xlnwid="", **kwargs):
    """Specifies annotation line attributes (GUI).

    APDL Command: /LSPEC

    Parameters
    ----------
    lcolor
        Line color:

        0 - Black

        1 - Red-Magenta

        2 - Magenta

        3 - Blue-Magenta

        4 - Blue

        5 - Cyan-Blue

        6 - Cyan

        7 - Green-Cyan

        8 - Green

        9 - Yellow-Green

        10 - Yellow

        11 - Orange

        12 - Red

        13 - Dark Gray

        14 - Light Gray

        15 - White

    linstl
        Line style:

        0 - Solid line.

        1 - Dashed line.

    xlnwid
        Line width multiplier (1.0 to 20.0).  Defaults to 1.0.

    Notes
    -----
    Specifies annotation line attributes to control certain characteristics
    of the lines created via the /LINE, /LARC, /LSYMBOL, /POLYGON, /PMORE,
    /PCIRCLE, and /PWEDGE commands.  This is a command generated by the
    Graphical User Interface (GUI) and will appear in the log file
    (Jobname.LOG) if annotation is used.  This command is not intended to
    be typed in directly in an ANSYS session (although it can be included
    in an input file for batch input or for use with the /INPUT command).

    This command is valid in any processor.
    """
    command = f"/LSPEC,{lcolor},{linstl},{xlnwid}"
    return self.run(command, **kwargs)


def lsymbol(self, x="", y="", symang="", symtyp="", symsiz="", keybmp="",
            **kwargs):
    """Creates annotation symbols (GUI).

    APDL Command: /LSYMBOL

    Parameters
    ----------
    x
        X location for symbol (-1.0 < X < 2.0).

    y
        Y location for symbol (-1.0 < Y < 1.0).

    symang
        Symbol orientation angle.

    symtyp
        Symbol type:

        1 - Arrow.

        2 - Tee.

        3 - Circle.

        4 - Triangle.

        5 - Star.

    symsiz
        Symbol size multiplier (0.1 to 20.0).  Defaults to 1.0.

    keybmp
        If KEYBMP = 1, the annotation is a bitmap. SYMTYP will then be a
        number from 1-99, indicating the bitmap type (see notes), and X and
        Y will define the lower left corner of the bitmap. The SYMANG,
        SYMSIZarguments are ignored. If KEYBMP = 0, or blank, then the
        argument definitions above apply.

    Notes
    -----
    Defines annotation symbols to be written directly onto the display at a
    specified location.  This is a command generated by the GUI and will
    appear in the log file (Jobname.LOG) if annotation is used.  This
    command is not intended to be typed in directly in an ANSYS session
    (although it can be included in an input file for batch input or for
    use with the /INPUT command).

    All symbols are shown on subsequent displays unless the annotation is
    turned off or deleted.  Use the /LSPEC command to set the attributes of
    the symbol.

    The KEYBMP argument reads the symtype argument to determine which
    bitmap to insert. This bitmap is defined by an integer between 1 and
    99. Numbers 1 through 40 correspond to the standard texture values
    found in the /TXTRE  command, while numbers 51 through 99 correspond to
    user supplied bitmaps, as defined using the Filename option of the
    /TXTRE command. Numbers 51 through 57 are predefined (the logos, clamps
    and arrows available from the GUI) but can be overridden. Numbers 41
    through 50 are reserved.

    This command is valid in any processor.
    """
    command = f"/LSYMBOL,{x},{y},{symang},{symtyp},{symsiz},{keybmp}"
    return self.run(command, **kwargs)


def pcircle(self, xcentr="", ycentr="", xlrad="", **kwargs):
    """Creates an annotation circle (GUI).

    APDL Command: /PCIRCLE

    Parameters
    ----------
    xcentr
        Circle X center location (-1.0 < X < 2.0).

    ycentr
        Circle Y center location (-1.0 < Y < 1.0).

    xlrad
        Circle radius length.

    Notes
    -----
    Creates an annotation circle to be written directly onto the display at
    a specified location.  This is a command generated by the Graphical
    User Interface (GUI) and will appear in the log file (Jobname.LOG) if
    annotation is used.  This command is not intended to be typed in
    directly in an ANSYS session (although it can be included in an input
    file for batch input or for use with the /INPUT command).

    All circles are shown on subsequent displays unless the annotation is
    turned off or deleted.  Use the /LSPEC and the /PSPEC command to set
    the attributes of the circle.

    This command is valid in any processor.
    """
    command = f"/PCIRCLE,{xcentr},{ycentr},{xlrad}"
    return self.run(command, **kwargs)


def pmore(self, x5="", y5="", x6="", y6="", x7="", y7="", x8="", y8="",
          **kwargs):
    """Creates an annotation polygon (GUI).

    APDL Command: /PMORE

    Parameters
    ----------
    x5
        X location for vertex 5 of polygon (-1.0 < X < 2.0).

    y5
        Y location for vertex 5 of polygon (-1.0 < Y < 1.0).

    x6
        X location for vertex 6 of polygon (-1.0 < X < 2.0).

    y6
        Y location for vertex 6 of polygon (-1.0 < Y < 1.0).

    x7
        X location for vertex 7 of polygon (-1.0 < X < 2.0).

    y7
        Y location for vertex 7 of polygon (-1.0 < Y < 1.0).

    x8
        X location for vertex 8 of polygon (-1.0 < X < 2.0).

    y8
        Y location for vertex 8 of polygon (-1.0 < Y < 1.0).

    Notes
    -----
    Defines the 5th through 8th vertices of an annotation polygon
    [/POLYGON].  This is a command generated by the Graphical User
    Interface (GUI) and will appear in the log file (Jobname.LOG) if
    annotation is used.  This command is not intended to be typed in
    directly in an ANSYS session (although it can be included in an input
    file for batch input or for use with the /INPUT command).

    This command is valid in any processor.
    """
    command = f"/PMORE,,{x5},{y5},{x6},{y6},{x7},{y7},{x8},{y8}"
    return self.run(command, **kwargs)


def polygon(self, nvert="", x1="", y1="", x2="", y2="", x3="", y3="",
            x4="", y4="", **kwargs):
    """Creates annotation polygons (GUI).

    APDL Command: /POLYGON

    Parameters
    ----------
    nvert
        Number of vertices of polygon (3  NVERT   8).  Use /PMORE for
        polygons with more than 4 vertices.

    x1
        X location for vertex 1 of polygon (-1.0 < X < 2.0).

    y1
        Y location for vertex 1 of polygon (-1.0 < Y < 1.0).

    x2
        X location for vertex 2 of polygon (-1.0 < X < 2.0).

    y2
        Y location for vertex 2 of polygon (-1.0 < Y < 1.0).

    x3
        X location for vertex 3 of polygon (-1.0 < X < 2.0).

    y3
        Y location for vertex 3 of polygon (-1.0 < Y < 1.0).

    x4
        X location for vertex 4 of polygon (-1.0 < X < 2.0).

    y4
        Y location for vertex 4 of polygon (-1.0 < Y < 1.0).

    Notes
    -----
    Creates annotation polygons to be written directly onto the display at
    a specified location.  This is a command generated by the Graphical
    User Interface (GUI) and will appear in the log file (Jobname.LOG) if
    annotation is used.  This command is not intended to be typed in
    directly in an ANSYS session (although it can be included in an input
    file for batch input or for use with the /INPUT command).

    All polygons are shown on subsequent displays unless the annotation is
    turned off or deleted.  Use the /LSPEC and the /PSPEC command to set
    the attributes of the polygon.  Use the /PMORE command to define the
    5th through 8th vertices of the polygon.

    This command is valid in any processor.
    """
    command = f"/POLYGON,{nvert},{x1},{y1},{x2},{y2},{x3},{y3},{x4},{y4}"
    return self.run(command, **kwargs)


def pspec(self, pcolor="", kfill="", kbordr="", **kwargs):
    """Creates annotation polygon attributes (GUI).

    APDL Command: /PSPEC

    Parameters
    ----------
    pcolor
        Polygon color (0  PCOLOR   15):

        0 - Black.

        1 - Red-Magenta.

        2 - Magenta.

        3 - Blue-Magenta.

        4 - Blue.

        5 - Cyan-Blue.

        6 - Cyan.

        7 - Green-Cyan.

        8 - Green.

        9 - Yellow-Green.

        10 - Yellow.

        11 - Orange.

        12 - Red.

        13 - Dark Gray.

        14 - Light Gray.

        15 - White.

    kfill
        Polygon fill key:

        0 - Hollow polygon.

        1 - Filled polygon.

    kbordr
        Polygon border key:

        0 - No border.

        1 - Border.

    Notes
    -----
    Creates annotation polygon attributes to control certain
    characteristics of the polygons created via the /POLYGON, /PMORE,
    /PCIRCLE and /PWEDGE commands.  This is a command generated by the
    Graphical User Interface (GUI) and will appear in the log file
    (Jobname.LOG) if annotation is used.  This command is not intended to
    be typed in directly in an ANSYS session (although it can be included
    in an input file for batch input or for use with the /INPUT command).

    This command is valid in any processor.
    """
    command = f"/PSPEC,{pcolor},{kfill},{kbordr}"
    return self.run(command, **kwargs)


def pwedge(self, xcentr="", ycentr="", xlrad="", angle1="", angle2="",
           **kwargs):
    """Creates an annotation wedge (GUI).

    APDL Command: /PWEDGE

    Parameters
    ----------
    xcentr
        Wedge X center location (-1.0 < X < 2.0).

    ycentr
        Wedge Y center location (-1.0 < Y < 1.0).

    xlrad
        Wedge radius length.

    angle1
        Starting angle of wedge.

    angle2
        Ending angle of wedge.  The wedge is drawn counterclockwise from
        the starting angle, ANGLE1, to the ending angle, ANGLE2.

    Notes
    -----
    Creates an annotation wedge to be written directly onto the display at
    a specified location.  This is a command generated by the Graphical
    User Interface (GUI) and will appear in the log file (Jobname.LOG) if
    annotation is used.  This command is not intended to be typed in
    directly in an ANSYS session (although it can be included in an input
    file for batch input or for use with the /INPUT command).

    All wedges are shown on subsequent displays unless the annotation is
    turned off or deleted.  Use the /LSPEC and the /PSPEC command to set
    the attributes of the wedge.

    This command is valid in any processor.
    """
    command = f"/PWEDGE,{xcentr},{ycentr},{xlrad},{angle1},{angle2}"
    return self.run(command, **kwargs)


def tlabel(self, xloc="", yloc="", text="", **kwargs):
    """Creates annotation text (GUI).

    APDL Command: /TLABEL

    Parameters
    ----------
    xloc
        Text X starting location (-1.0 < X < 1.6).

    yloc
        Text Y starting location (-1.0 < Y < 1.0).

    text
        Text string (60 characters maximum).  Parameter substitution may be
        forced within the text by enclosing the parameter name or
        parametric expression within percent (%) signs.

    Notes
    -----
    Defines annotation text to be written directly onto the display at a
    specified location.  This is a command generated by the Graphical User
    Interface (GUI) and will appear in the log file (Jobname.LOG) if
    annotation is used.  This command is not intended to be typed in
    directly in an ANSYS session (although it can be included in an input
    file for batch input or for use with the /INPUT command).

    All text is shown on subsequent displays unless the annotation is
    turned off or deleted.  Use the /TSPEC command to set the attributes of
    the text.

    This command is valid in any processor.
    """
    command = f"/TLABEL,{xloc},{yloc},{text}"
    return self.run(command, **kwargs)


def tspec(self, tcolor="", tsize="", txthic="", pangle="", iangle="",
          **kwargs):
    """Creates annotation text attributes (GUI).

    APDL Command: /TSPEC

    Parameters
    ----------
    tcolor
        Text color (0  TCOLOR   15):

        0 - Black.

        1 - Red-Magenta.

        2 - Magenta.

        3 - Blue-Magenta.

        4 - Blue.

        5 - Cyan-Blue.

        6 - Cyan.

        7 - Green-Cyan.

        8 - Green.

        9 - Yellow-Green.

        10 - Yellow.

        11 - Orange.

        12 - Red.

        13 - Dark Gray.

        14 - Light Gray.

        15 - White.

    tsize
        Text size factor.

    txthic
        Text thickness key:

        1 - normal.

        2 - twice as thick.

        3 - three times as thick.

        4 - four times as thick.

    pangle
        Text path angle (0.0 < angle < 360.0).

    iangle
        Text italic angle (0.0 < angle < 45.0).

    Notes
    -----
    Defines annotation text attributes to control certain characteristics
    of the text created via the /TLABEL command.  This is a command
    generated by the Graphical User Interface (GUI) and will appear in the
    log file (Jobname.LOG) if annotation is used.  This command is not
    intended to be typed in directly in an ANSYS session (although it can
    be included in an input file for batch input or for use with the /INPUT
    command).

    This command is valid in any processor.
    """
    command = f"/TSPEC,{tcolor},{tsize},{txthic},{pangle},{iangle}"
    return self.run(command, **kwargs)



def pmlopt(self, esys="", lab="", xminus="", xplus="", yminus="", yplus="",
           zminus="", zplus="", **kwargs):
    """Defines perfectly matched layers (PMLs) for acoustic and structural

    APDL Command: PMLOPT
    analyses.

    Parameters
    ----------
    esys
        Element coordinate system number. ESYS may be 0 (global Cartesian)
        or any previously defined local Cartesian coordinate system number
        (>10). Defaults to 0.

    lab
        Label defining the number of dimensions:

        ONE - A one-dimensional PML region.

        THREE - A three-dimensional PML region (default).

    xminus
        For harmonic analysis, normal reflection coefficient in negative X
        direction of ESYS. Defaults to 1.E−3 (equivalent to -60 dB) for a
        harmonic analysis and 30 for a static structural analysis.

    xplus
        For harmonic analysis, normal reflection coefficient in positive X
        direction of ESYS. Defaults to 1.E−3 (equivalent to -60 dB) for a
        harmonic analysis and 30 for a static structural analysis.

    yminus
        For harmonic analysis, normal reflection coefficient in negative Y
        direction of ESYS. Defaults to 1.E−3 (equivalent to -60 dB) for a
        harmonic analysis and 30 for a static structural analysis.

    yplus
        For harmonic analysis, normal reflection coefficient in positive Y
        direction of ESYS. Defaults to 1.E−3 (equivalent to -60 dB) for a
        harmonic analysis and 30 for a static structural analysis.

    zminus
        For harmonic analysis, normal reflection coefficient in negative Z
        direction of ESYS. Defaults to 1.E−3 (equivalent to -60 dB) for a
        harmonic analysis and 30 for a static structural analysis.

    zplus
        For harmonic analysis, normal reflection coefficient in positive Z
        direction of ESYS. Defaults to 1.E−3 (equivalent to -60 dB) for a
        harmonic analysis and 30 for a static structural analysis.

    Notes
    -----
    PMLOPT defines perfectly matched layers (PML) for acoustic or
    structural analyses. Each PML region must have a uniquely defined
    element coordinate system. Normal reflection coefficient values for a
    harmonic analysis must be less than 1.

    Issue PMLOPT,STAT to list the current normal reflection coefficient or
    attenuation factor settings for a PML region. Issue PMLOPT,CLEAR to
    clear all normal reflection coefficient settings and restore them to
    the defaults. Issue PMLOPT,ESYS,CLEAR to clear all normal reflection
    coefficient settings for this element coordinate system and restore
    them to the defaults.
    """
    command = f"PMLOPT,{esys},{lab},{xminus},{xplus},{yminus},{yplus},{zminus},{zplus}"
    return self.run(command, **kwargs)


def pmlsize(self, freqb="", freqe="", dmin="", dmax="", thick="", angle="",
            **kwargs):
    """Determines number of PML layers.

    APDL Command: PMLSIZE

    Parameters
    ----------
    freqb
        Minimum operating frequency

    freqe
        Maximum operating frequency

    dmin
        Minimum distance from radiation source to PML interface.

    dmax
        Maximum distance from radiation source to PML interface.

    thick
        Thickness of PML region. Defaults to 0.

    angle
        Incident angle of wave to the PML interface. Defaults to 0.

    Notes
    -----
    PMLSIZE determines the number of PML layers for acceptable numerical
    accuracy.

    PMLSIZE must be issued before any meshing commands. If the thickness of
    the PML region is known, it determines an element edge length (h) and
    issues ESIZE,h.  If the thickness of the PML region is unknown, it
    determines the number of layers (n) and issues ESIZE,,n.
    """
    command = f"PMLSIZE,{freqb},{freqe},{dmin},{dmax},{thick},{angle}"
    return self.run(command, **kwargs)



def da(self, area="", lab="", value1="", value2="", **kwargs):
    """Defines degree-of-freedom constraints on areas.

    APDL Command: DA

    Parameters
    ----------
    area
        Area on which constraints are to be specified.  If ALL, apply to
        all selected areas [ASEL].  If AREA = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for AREA.

    lab
        Symmetry label (see 2 below):

        SYMM - Generate symmetry constraints. Requires no Value1 or Value2.

        ASYM - Generate antisymmetry constraints. Requires no Value1 or Value2.

    value1
        Value of DOF or table name reference on the area.  Valid for all
        DOF labels.  To specify a table, enclose the table name in % signs
        (e.g., DA,AREA,TEMP,%tabname%).  Use the *DIM command to define a
        table.

    value2
        For MAG and VOLT DOFs:

    Notes
    -----
    For elements SOLID236 and SOLID237, if Lab = AZ and Value1 = 0, this
    sets the flux-parallel condition for the edge formulation.  (A flux-
    normal condition is the natural boundary condition.)  Do not use the DA
    command to set the edge-flux DOF, AZ to a nonzero value.

    If Lab = MAG and Value1 = 0, this sets the flux-normal condition for
    the magnetic scalar potential formulations (MSP) (A flux-parallel
    condition is the natural boundary condition for MSP.)

    If Lab = VOLT and Value1 = 0, the J-normal condition is set (current
    density (J) flow normal to the area). (A J-parallel condition is the
    natural boundary condition.)

    You can transfer constraints from areas to nodes with the DTRAN or
    SBCTRAN commands.  See the DK command for information about generating
    other constraints on areas.

    Symmetry and antisymmetry constraints are generated as described for
    the DSYM command.

    Tabular boundary conditions (VALUE = %tabname%) are available only for
    the following degree of freedom labels: Electric (VOLT), Structural
    (UX, UY, UZ, ROTX, ROTY, ROTZ), Acoustic (PRES, UX, UY, UZ), and
    temperature (TEMP, TBOT, TE2, TE3, . . ., TTOP).

    Constraints specified by the DA command can conflict with other
    specified constraints.  See Resolution of Conflicting Constraint
    Specifications in the Basic Analysis Guide for details.

    The DA command is also valid in PREP7.
    """
    command = f"DA,{area},{lab},{value1},{value2}"
    return self.run(command, **kwargs)


def dadele(self, area="", lab="", **kwargs):
    """Deletes degree-of-freedom constraints on an area.

    APDL Command: DADELE

    Parameters
    ----------
    area
        Area for which constraints are to be deleted.  If ALL, delete for
        all selected areas [ASEL].  If AREA = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  You can substitute a component name  for AREA.

    lab
        Valid constraint labels are:

        ALL - All constraints.

        SYMM - Symmetry constraints.

        ASYM - Antisymmetry constraints.

        UX - Displacement in X direction.

        UY - Displacement in Y direction.

        UZ - Displacement in Z direction.

        ROTX - Rotation about X axis.

        ROTY - Rotation about Y axis.

        ROTZ - Rotation about Z axis.

        PRES - Pressure.

        TEMP, TBOT, TE2, TE3, . . ., TTOP - Temperature.

        MAG - Magnetic scalar potential.

        VOLT - Electric scalar potential.

        AX - Magnetic vector potential in X direction (see notes).

        AY - Magnetic vector potential in Y direction.

        AZ - Magnetic vector potential in Z direction (see notes).

        CONC - Concentration.

    Notes
    -----
    Deletes the degree of freedom constraints at an area (and all
    corresponding finite element constraints) previously specified with the
    DA command. See the DDELE command for delete details.

    If the multiple species labels have been changed to user-defined labels
    via the MSSPEC command, use the user-defined labels.

    See the DA or the DA commands for details on element applicability.

    Warning:: : On previously meshed areas, all constraints on affected
    nodes will be deleted, whether or not they were specified by the DA
    command.

    This command is also valid in PREP7.
    """
    command = f"DADELE,{area},{lab}"
    return self.run(command, **kwargs)


def dalist(self, area="", **kwargs):
    """Lists the DOF constraints on an area.

    APDL Command: DALIST

    Parameters
    ----------
    area
        List constraints for this area.  If ALL (default), list for all
        selected areas [ASEL].  If P1 = P, graphical picking is enabled and
        all remaining command fields are ignored (valid only in the GUI).
        A component name may also be substituted for AREA.

    Notes
    -----
    Lists the degree of freedom constraints on an area previously specified
    with the DA command.

    This command is valid in any processor.
    """
    command = f"DALIST,{area}"
    return self.run(command, **kwargs)


def dk(self, kpoi="", lab="", value="", value2="", kexpnd="", lab2="",
       lab3="", lab4="", lab5="", lab6="", **kwargs):
    """Defines DOF constraints at keypoints.

    APDL Command: DK

    Parameters
    ----------
    kpoi
        Keypoint at which constraint is to be specified.  If ALL, apply to
        all selected keypoints [KSEL].  If KPOI = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for KPOI.

    lab
        Valid degree of freedom label.  If ALL, use all appropriate labels
        except HDSP.  Structural labels:  UX, UY, or UZ (displacements);
        ROTX, ROTY, or ROTZ (rotations); WARP (warping); HDSP (hydrostatic
        pressure).  Thermal labels: TEMP, TBOT, TE2, TE3, . . ., TTOP
        (temperature). Acoustic labels:  PRES (pressure); UX, UY, or UZ
        (displacements for FSI coupled elements). Electric labels:  VOLT
        (voltage). Magnetic labels:  MAG (scalar magnetic potential); AX,
        AY, or AZ (vector magnetic potentials). Diffusion labels: CONC
        (concentration).

    value
        Degree of freedom value or table name reference for tabular
        boundary conditions.  To specify a table, enclose the table name in
        percent signs (%) (e.g., DK,NODE,TEMP,%tabname%).  Use the *DIM
        command to define a table.

    value2
        Second degree of freedom value (if any).  If the analysis type and
        the degree of freedom allow a complex input, VALUE (above) is the
        real component and VALUE2 is the imaginary component.

    kexpnd
        Expansion key:

        0 - Constraint applies only to the node at this keypoint.

        1 - Flags this keypoint for constraint expansion.

    lab2, lab3, lab4, . . . ,  lab6
        Additional degree of freedom labels.  The same values are applied
        to the keypoints for these labels.

    Notes
    -----
    A keypoint may be flagged using KEXPND to allow its constraints to be
    expanded to nodes on the attached solid model entities having similarly
    flagged keypoint constraints.  Constraints are transferred from
    keypoints to nodes with the DTRAN or SBCTRAN commands.  The expansion
    uses interpolation to apply constraints to the nodes on the lines
    between flagged keypoints.  If all keypoints of an area or volume
    region are flagged and the constraints (label and values) are equal,
    the constraints are applied to the interior nodes of the region.  See
    the D command for a description of nodal constraints.

    Tabular boundary conditions (VALUE = %tabname%) are available only for
    the following degree of freedom labels: Electric (VOLT), structural
    (UX, UY, UZ, ROTX, ROTY, ROTZ), Acoustic (PRES, UX, UY, UZ), and
    temperature (TEMP, TBOT, TE2, TE3, . . ., TTOP).

    Constraints specified by the DK command can conflict with other
    specified constraints.  See Resolution of Conflicting Constraint
    Specifications in the Basic Analysis Guide for details.

    This command is also valid in PREP7.
    """
    command = f"DK,{kpoi},{lab},{value},{value2},{kexpnd},{lab2},{lab3},{lab4},{lab5},{lab6}"
    return self.run(command, **kwargs)


def dkdele(self, kpoi="", lab="", **kwargs):
    """Deletes DOF constraints at a keypoint.

    APDL Command: DKDELE

    Parameters
    ----------
    kpoi
        Keypoint for which constraint is to be deleted.  If ALL, delete for
        all selected keypoints [KSEL].  If KPOI = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for KPOI.

    lab
        Valid degree of freedom label.  If ALL, use all appropriate labels.
        Structural labels:  UX, UY, or UZ (displacements); ROTX, ROTY, or
        ROTZ (rotations); WARP (warping).  Thermal labels: TEMP, TBOT, TE2,
        TE3, . . ., TTOP (temperature). Acoustic labels:  PRES (pressure);
        UX, UY, or UZ (displacements for FSI coupled elements). Electric
        label:  VOLT (voltage).  Magnetic labels:  MAG (scalar magnetic
        potential); AX, AY, or AZ (vector magnetic potentials).  Diffusion
        label: CONC (concentration).

    Notes
    -----
    Deletes the degree of freedom constraints (and all corresponding finite
    element constraints) at a keypoint.  See the DDELE command for details.

    This command is also valid in PREP7.
    """
    command = f"DKDELE,{kpoi},{lab}"
    return self.run(command, **kwargs)


def dklist(self, kpoi="", **kwargs):
    """Lists the DOF constraints at keypoints.

    APDL Command: DKLIST

    Parameters
    ----------
    kpoi
        List constraints for this keypoint.  If ALL (default), list for all
        selected keypoints [KSEL].  If KPOI = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for KPOI.

    Notes
    -----
    Listing applies to the selected keypoints [KSEL] and the selected
    degree of freedom labels [DOFSEL].

    This command is valid in any processor.
    """
    command = f"DKLIST,{kpoi}"
    return self.run(command, **kwargs)


def dl(self, line="", area="", lab="", value1="", value2="", **kwargs):
    """Defines DOF constraints on lines.

    APDL Command: DL

    Parameters
    ----------
    line
        Line at which constraints are to be specified. If ALL, apply to all
        selected lines [LSEL]. If LINE = P, graphical picking is enabled
        and all remaining command fields are ignored (valid only in the
        GUI). A component name may also be substituted for LINE.

    area
        Area containing line. The normal to the symmetry or antisymmetry
        surface is assumed to lie on this area. Defaults to the lowest
        numbered selected area containing the line number.

    lab
        Symmetry label (see 2):

        SYMM - Generate symmetry constraints.

        ASYM - Generate antisymmetry constraints.

    value1
        Value of DOF (real part) or table name reference on the line.
        Valid for all DOF labels.  To specify a table, enclose the table
        name in % signs (e.g., DL,LINE,AREA,TEMP,%tabname%).  Use the *DIM
        command to define a table.

    value2
        For VOLT DOFs:

    Notes
    -----
    You can transfer constraints from lines  to nodes with the  DTRAN or
    SBCTRAN commands.  See the DK command for information about generating
    other constraints at lines.

    Symmetry and antisymmetry constraints are generated as described on the
    DSYM command.

    Setting Lab = VOLT and Value1 = 0 applies the J-normal boundary
    condition (current density vector (J) flows normal to the line).  No
    input is required for the J-parallel condition because it is the
    natural boundary condition.

    Tabular boundary conditions (Value1 = %tabname%) are available only for
    the following degree of freedom labels: Electric (VOLT), Structural
    (UX, UY, UZ, ROTX, ROTY, ROTZ), Acoustic (PRES, UX, UY, UZ), and
    temperature (TEMP, TBOT, TE2, TE3, . . ., TTOP).

    Constraints specified by the DL command can conflict with other
    specified constraints.  See Resolution of Conflicting Constraint
    Specifications in the Basic Analysis Guide for details.

    This command is also valid in PREP7.
    """
    command = f"DL,{line},{area},{lab},{value1},{value2}"
    return self.run(command, **kwargs)


def dldele(self, line="", lab="", **kwargs):
    """Deletes DOF constraints on a line.

    APDL Command: DLDELE

    Parameters
    ----------
    line
        Line for which constraints are to be deleted.  If ALL, delete for
        all selected lines [LSEL].  If LINE = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for LINE

    lab
        Constraint label:

        ALL - All constraints.

        SYMM - Symmetry constraints.

        ASYM - Antisymmetry constraints.

        UX - Displacement in X direction.

        UY - Displacement in Y direction.

        UZ - Displacement in Z direction.

        ROTX - Rotation about X axis.

        ROTY - Rotation about Y axis.

        ROTZ - Rotation about Z axis.

        WARP - Warping magnitude.

        PRES - Pressure.

        TEMP, TBOT, TE2, TE3, . . ., TTOP - Temperature.

        VOLT - Electric scalar potential.

        AX - Magnetic vector potential in X direction.

        AY - Magnetic vector potential in Y direction.

        AZ - Magnetic vector potential in Z direction.

        CONC - Concentration.

    Notes
    -----
    Deletes the degree of freedom constraints (and all corresponding finite
    element constraints) on a line previously specified with the DL
    command.  See the DDELE command for delete details.

    Warning:: : On previously meshed lines, all constraints on affected
    nodes will also be deleted, whether or not they were specified by the
    DL command.

    This command is also valid in PREP7.
    """
    command = f"DLDELE,{line},{lab}"
    return self.run(command, **kwargs)


def dllist(self, line="", **kwargs):
    """Lists DOF constraints on a line.

    APDL Command: DLLIST

    Parameters
    ----------
    line
        List constraints for this line.  If ALL (default), list for all
        selected lines [LSEL].  If LINE = P, graphical picking is enabled
        and all remaining command fields are ignored (valid only in the
        GUI).  A component name may also be substituted for LINE.

    Notes
    -----
    Lists the degree of freedom constraints on a line previously specified
    with the DL command.

    This command is valid in any processor.
    """
    command = f"DLLIST,{line}"
    return self.run(command, **kwargs)


def dtran(self, **kwargs):
    """Transfers solid model DOF constraints to the finite element model.

    APDL Command: DTRAN

    Notes
    -----
    Constraints are transferred only from selected solid model entities to
    selected nodes.  The DTRAN operation is also done if the SBCTRAN
    command is issued, and is automatically done upon initiation of the
    solution calculations [SOLVE].

    This command is also valid in PREP7.
    """
    command = f"DTRAN,"
    return self.run(command, **kwargs)



"""These DATABASE commands allow selected subsets of entities to be
named as components for easy selection later on.
"""


def cm(self, cname="", entity="", **kwargs):
    """Groups geometry items into a component.

    APDL Command: CM

    Parameters
    ----------
    cname
        An alphanumeric name used to identify this component. Cname may be
        up to 32 characters, beginning with a letter and containing only
        letters, numbers, and underscores.  Component names beginning with
        an underscore (e.g., _LOOP) are reserved for use by ANSYS and
        should be avoided.  Components named "ALL," "STAT," and "DEFA" are
        not permitted. Overwrites a previously defined name.

    entity
        Label identifying the type of geometry items to be grouped:

        VOLU - Volumes.

        AREA - Areas.

        LINE - Lines.

        KP - Keypoints.

        ELEM - Elements.

        NODE - Nodes.

    Notes
    -----
    Components may be further grouped into assemblies [CMGRP].  The
    selected items of the specified entity type will be stored as the
    component.  Use of this component in the select command [CMSEL] causes
    all these items to be selected at once, for convenience.

    A component is a grouping of some geometric entity that can then be
    conveniently selected or unselected.  A component may be redefined by
    reusing a previous component name.  The following entity types may
    belong to a component: nodes, elements, keypoints, lines, areas, and
    volumes.  A component may contain only 1 entity type, but an individual
    item of any entity may belong to any number of components.  Once
    defined, the items contained in a component may then be easily selected
    or unselected [CMSEL].  Components may be listed [CMLIST], modified
    [CMMOD] and deleted [CMDELE].  Components may also be further grouped
    into assemblies [CMGRP].  Other entities associated with the entities
    in a component (e.g., the lines and keypoints associated with areas)
    may be selected by the ALLSEL command.

    An item will be deleted from a component if it has been deleted by
    another operation (see the KMODIF command for an example).  Components
    are automatically updated to reflect deletions of one or more of their
    items.  Components are automatically deleted and a warning message is
    issued if all their items are deleted.  Assemblies are also
    automatically updated to reflect deletions of one or more of their
    components or subassemblies, but are not deleted if all their
    components and subassemblies are deleted.

    This command is valid in any processor.
    """
    command = "CM,%s,%s" % (str(cname), str(entity))
    return self.run(command, **kwargs)


def cmdele(self, name="", **kwargs):
    """Deletes a component or assembly definition.

    APDL Command: CMDELE

    Parameters
    ----------
    name
        Name of the component or assembly whose definition is to be
        removed.

    Notes
    -----
    Entities contained in the component, or the components within the
    assembly, are unaffected.  Only the grouping relationships are deleted.
    Assemblies are automatically updated to reflect deletion of their
    components or subassemblies, but they are not automatically deleted
    when all their components or subassemblies are deleted.

    This command is valid in any processor.
    """
    command = "CMDELE,%s" % (str(name))
    return self.run(command, **kwargs)


def cmedit(self, aname="", oper="", cnam1="", cnam2="", cnam3="", cnam4="",
           cnam5="", cnam6="", cnam7="", **kwargs):
    """Edits an existing assembly.

    APDL Command: CMEDIT

    Parameters
    ----------
    aname
        Name of the assembly to be edited.

    oper
        Operation label:

        ADD - To add more components.  The level of any assembly to be added must be lower
              than that of the assembly Aname (see CMGRP command).

        DELE - To remove components.

    cnam1, cnam2, cnam3, . . . , cnam7
        Names of components and assemblies to be added to or deleted from
        the assembly.

    Notes
    -----
    This command is valid in any processor.
    """
    command = "CMEDIT,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(aname), str(oper), str(
        cnam1), str(cnam2), str(cnam3), str(cnam4), str(cnam5), str(cnam6), str(cnam7))
    return self.run(command, **kwargs)


def cmgrp(self, aname="", cnam1="", cnam2="", cnam3="", cnam4="", cnam5="",
          cnam6="", cnam7="", cnam8="", **kwargs):
    """Groups components and assemblies into an assembly.

    APDL Command: CMGRP

    Parameters
    ----------
    aname
        An alphanumeric name used to identify this assembly.  Aname may be
        up to 32 characters, beginning with a letter and containing only
        letters, numbers, and underscores.  Overwrites a previously defined
        Aname (and removes it from higher level assemblies, if any).

    cnam1, cnam2, cnam3, . . . , cnam8
        Names of existing components or other assemblies to be included in
        this assembly.

    Notes
    -----
    Groups components and other assemblies into an assembly identified by a
    name.  CMGRP is used for the initial definition of an assembly.  An
    assembly is used in the same manner as a component.  Up to 5 levels of
    assemblies within assemblies may be used.

    An assembly is a convenient grouping of previously defined components
    and other assemblies.  Assemblies may contain components only, other
    assemblies, or any combination.  A component may belong to any number
    of assemblies.  Up to 5 levels of nested assemblies may be defined.
    Components and assemblies may be added to or deleted from an existing
    assembly by the CMEDIT command.  Once defined, an assembly may be
    listed, deleted, selected, or unselected using the same commands as for
    a component.  Assemblies are automatically updated to reflect deletions
    of one or more of their components or lower-level assemblies.
    Assemblies are not automatically deleted when all their components or
    subassemblies are deleted.

    This command is valid in any processor.
    """
    command = "CMGRP,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(aname), str(cnam1), str(
        cnam2), str(cnam3), str(cnam4), str(cnam5), str(cnam6), str(cnam7), str(cnam8))
    return self.run(command, **kwargs)


def cmlist(self, name="", key="", entity="", **kwargs):
    """Lists the contents of a component or assembly.

    APDL Command: CMLIST

    Parameters
    ----------
    name
        Name of the component or assembly to be listed (if blank, list all
        selected components and assemblies). If Name is specified, then
        Entity  is ignored.

    key
        Expansion key:

        0 - Do not list individual entities in the component.

        1 or EXPA - List individual entities in the component.

    entity
        If Name is blank, then the following entity types can be specified:

        VOLU - List the volume components only.

        AREA - List the area components only.

        LINE - List the line components only.

        KP - List the keypoint components only

        ELEM - List the element components only.

        NODE - List the node components only.

    Notes
    -----
    This command is valid in any processor.  For components, it lists the
    type of geometric entity. For assemblies, it lists the components
    and/or assemblies that make up the assembly.

    Examples of possible usage:
    """
    command = "CMLIST,%s,%s,%s" % (str(name), str(key), str(entity))
    return self.run(command, **kwargs)


def cmmod(self, cname="", keyword="", value="", **kwargs):
    """Modifies the specification of a component.

    APDL Command: CMMOD

    Parameters
    ----------
    cname
        Name of the existing component or assembly to be modified.

    keyword
        The label identifying the type of value to be modified.

    value
        If Keyword is NAME, then the value is the alphanumeric label to be
        applied. See the CM command for naming convention details. If a
        component named Value already exists, the command will be ignored
        and an error message will be generated.

    Notes
    -----
    The naming conventions for components, as specified in the CM command,
    apply for CMMOD (32 characters, "ALL", "STAT" and "DEFA" are not
    allowed, etc.). However, if you choose a component name that is already
    designated for another component, an error message will be issued and
    the command will be ignored.

    This command is valid in any processor.
    """
    command = "CMMOD,%s,%s,%s" % (str(cname), str(keyword), str(value))
    return self.run(command, **kwargs)


def cmplot(self, label="", entity="", keyword="", **kwargs):
    """Plots the entities contained in a component or assembly.

    APDL Command: CMPLOT

    Parameters
    ----------
    label
        Name of the component or assembly to be plotted.

        (blank) - All selected components and assemblies are plotted (default).  If fewer than 11
                  components are selected, then all  are plotted.  If more
                  than 11 components are selected, then only the first 11
                  are plotted.

        ALL - All selected components are plotted. If number of selected components is
              greater than 11, then the legend showing component names will
              not be shown.

        N - Next set of defined components and assemblies is plotted.

        P - Previous set of defined components and assemblies is plotted.

        Cname - The specified component or assembly is plotted.

        SetNo. - The specified set number is plotted.

    entity
        If Label is BLANK or ALL, then the following entity types can be
        specified:

        VOLU - Plot the volume components only.

        AREA - Plot the area components only.

        LINE - Plot the line components only.

        KP - Plot the keypoint components only.

        ELEM - Plot the element components only.

        NODE - Plot the node components only.

    keyword
        For Keyword = ALL, plot the specified component name in the Label
        field in the context of all entities of the same type. Not valid if
        Label field is BLANK or ALL.

    Notes
    -----
    Components are plotted with their native entities.  For assemblies, all
    native entities for the underlying component types are plotted
    simultaneously.  Although more components can be plotted, the legend
    displays only 11 at a time. When more than eleven are plotted, the
    legend is not displayed.

    Possible usage:

    This command is valid in any processor.
    """
    command = "CMPLOT,%s,%s,%s" % (str(label), str(entity), str(keyword))
    return self.run(command, **kwargs)


def cmsel(self, type_="", name="", entity="", **kwargs):
    """Selects a subset of components and assemblies.

    APDL Command: CMSEL

    Parameters
    ----------
    type\_
        Label identifying the type of select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

        ALL - Also select all components.

        NONE - Unselect all components.

    name
        Name of component or assembly whose items are to be selected (valid
        only if Type = S, R, A, or U).

    entity
        If Name is blank, then the following entity types can be specified:

        VOLU - Select the volume components only.

        AREA - Select the area components only.

        LINE - Select the line components only.

        KP - Select the keypoint components only.

        ELEM - Select the element components only.

        NODE - Select the node components only.

    Notes
    -----
    Selecting by component is a convenient adjunct to individual item
    selection (e.g., VSEL, ESEL, etc.). CMSEL, ALL allows you to select
    components in addition to other items you have already selected.

    If Type = R for an assembly selection [CMSEL,R,<assembly-name>], the
    reselect operation is performed on each component in the assembly in
    the order in which the components make up the assembly.  Thus, if one
    reselect operation results in an empty set, subsequent operations will
    also result in empty sets.  For example, if the first reselect
    operation tries to reselect node 1 from the selected set of nodes 3, 4,
    and 5, the operation results in an empty set (that is, no nodes are
    selected).  Since the current set is now an empty set, if the second
    reselect operation tries to reselect any nodes, the second operation
    also results in an empty set, and so on.  This is equivalent to
    repeating the command CMSEL,R,<component-name> once for each component
    making up the assembly.

    This command is valid in any processor.
    """
    command = "CMSEL,%s,%s,%s" % (str(type_), str(name), str(entity))
    return self.run(command, **kwargs)


def cmwrite(self, option="", fname="", ext="", fnamei="", exti="",
            fmat="", **kwargs):
    """Writes node and element components and assemblies to a file.

    APDL Command: CMWRITE

    Parameters
    ----------
    option
        Selects which data to write:

        ALL - Write all appropriate geometry, material property,
              load, and component data (default). Two files will
              be produced. Fname.Ext will contain all data items
              mentioned in "Notes", except the solid model
              data. Fnamei.Exti will contain the solid model
              geometry and solid model loads data in the form of
              IGES commands. This option is not valid when
              CDOPT,ANF is active.

        COMB - Write all data mentioned, but to a single file,
               Fname.Ext. Solid model geometry data will be
               written in either IGES or ANF format as specified
               in the CDOPT command, followed by the remainder of
               the data in the form of ANSYS commands. More
               information on these (IGES/ANF) file formats is
               provided in "Notes".

        DB - Write all database information except the solid model
             and solid model loads to Fname.Ext in the form of
             ANSYS commands. This option is not valid when
             CDOPT,ANF is active.

        SOLID - Write only the solid model geometry and solid
                model load data. This output will be in IGES or
                ANF format, as specified in the CDOPT
                command. More information on these (IGES/ANF) file
                formats is provided in "Notes".

        GEOM - Write only element and nodal geometry data. Neither
               solid model geometry nor element attribute data
               will be written. One file, Fname.Ext, will be
               produced. Use CDREAD,DB to read in a file written
               with this option. Element types [ET] compatible
               with the connectivity of the elements on the file
               must first be defined before reading the file in
               with CDREAD,DB.

        CM - Write only node and element component and geometry
             data to Fname.Ext.

        MAT - Write only material property data (both linear and
              nonlinear) to Fname.Ext .

        LOAD - Write only loads for current load step to
               Fname.Ext.

        SECT - Write only section data to Fname.Ext. Pretension
               sections are not included.


    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).  The
        extension defaults to CDB if Fname is blank.

    fnamei
        Name of the IGES file and its directory path (248
        characters maximum, including directory). If you do not
        specify a directory path, it will default to your working
        directory and you can use all 248 characters for the file
        name.

        The file name defaults to Fname. Used only if
        Option = ALL or SOLID. Previous data on this file, if any,
        is overwritten.

    Exti
        Filename extension (eight-character maximum).  The
        extension defaults to IGES in all cases, except when
        CDOPT,ANF is active and CDWRITE, Option = SOLID. In this
        case Exti = ANF.

    fmat
        Format of the output file (defaults to BLOCKED).

        BLOCKED - Blocked format. This format allows faster
                  reading of the output file. The time savings is
                  most significant when BLOCKED is used to read
                  .cdb files associated with very large models.

        UNBLOCKED - Unblocked format.

    Notes
    -----
    Load data includes the current load step only. Loads applied
    to the solid model (if any) are automatically transferred to
    the finite element model when this command is issued. CDWRITE
    writes out solid model loads for meshed models only. If the
    model is not meshed, the solid model loads cannot be
    saved. Component data include component definitions, but not
    assembly definitions. Appropriate NUMOFF commands are included
    at the beginning of the file; this is to avoid overlap of an
    existing database when the file is read in.

    Solution control commands are typically not written to the
    file unless you specifically change a default solution
    setting.

    CDWRITE does not support the GSBDATA and GSGDATA commands, and
    these commands are not written to the file.

    The data may be reread (on a different machine, for example)
    with the CDREAD command. Caution: When the file is read in,
    the NUMOFF,MAT command may cause a mismatch between material
    definitions and material numbers referenced by certain loads
    and element real constants. See NUMOFF for details. Also, be
    aware that the files created by the CDWRITE command explicitly
    set the active coordinate system to Cartesian (CSYS,0).

    You should generally use the blocked format (Fmat = BLOCKED)
    when writing out model data with CDWRITE. This is a compressed
    data format that greatly reduces the time required to read
    large models through the CDREAD command. The blocked and
    unblocked formats are described in Chapter 3 of the Guide to
    Interfacing with ANSYS.

    If you use CDWRITE in any of the derived products (ANSYS
    Mechanical Pro, ANSYS Mechanical Premium), then before reading
    the file, you must edit the Jobname.cdb file to remove
    commands that are not available in the respective component
    product.

    The CDWRITE command writes PART information for any ANSYS
    LS-DYNA input file to the Jobname.cdb file via the EDPREAD
    command. (EDPREAD is not a documented command; it is written
    only when the CDWRITE command is issued.) The PART information
    can be automatically read in via the CDREAD command; however,
    if more than one Jobname.cdb file is read, the PART list from
    the last Jobname.cdb file overwrites the existing PART list of
    the total model. This behavior affects all PART-related
    commands contained in the Jobname.cdb file. You can join
    models, but not PART-related inputs, which you must modify
    using the newly-created PART numbers. In limited cases, an
    update of the PART list (EDWRITE,PUPDATE) is possible; doing
    so requires that no used combination of MAT/TYPE/REAL appears
    more than once in the list.

    The CDWRITE command does not support (for beam meshing) any
    line operation that relies on solid model associativity. For
    example, meshing the areas adjacent to the meshed line,
    plotting the line that contains the orientation nodes, or
    clearing the mesh from the line that contains orientation
    nodes may not work as expected. For more information about
    beam meshing, see Meshing Your Solid Model in the Modeling and
    Meshing Guide.
    """
    command = f"CDWRITE,{option},{fname},{ext},,{fnamei},{exti},{fmat}"
    return self.run(command, **kwargs)
